/*  ********************************************************************
    ASPTADJLMS.H 

    Declares the Adjoint LMS algorithm.
    This version is intended for simulation purposes and matches 
    the Matlab asptadjlms and init_adjlms functions.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTADJLMS_H_
#define ASPTADJLMS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon32f.h"

typedef struct _asptAdjlms32f {
	DT32S	filterLength;			/* filter length in samples   */
	DT32S	secPathLength;			/* secondary path length      */
	DT32S	estSecPathLength;		/* estimated sec. path length */
	DT32S	refLength;				/* length of reference signal */
	DT32F	stepSize;				/* LMS step size              */
	DT32F	sensor;					/* last sensor sample         */
	DT32F	power;                  /* last estimated input power */
	DT32F	*coef;					/* filter coef. vector        */
	DT32F	*secPath;				/* secondary path             */
	DT32F	*secPathEst;			/* estimated secondary path   */
	DT32F	*yBuf;					/* buffer for output signal   */
	DT32F	*feBuf;					/* filtered-error signal      */
	DT32F	*delayLine;				/* pointer to the filter data */
	DT32S	xIndex;
	DT32S	yIndex;
	DT32S	eIndex;
} asptAdjlms32f;

/* API to create and delete an asptAdjlms32f filters */
DT32S	adjlms32fInit (asptAdjlms32f *adjlms,DT32S L, DT32F mu, DT32F *sPath, DT32S sL, DT32F *sPathEst, DT32S seL);
DT32S	adjlms32fInitStatic(asptAdjlms32f *adjlms,DT32S L, DT32F mu, DT32F *sPath, DT32S sL, DT32F *sPathEst, DT32S seL, DT32F *pCof, DT32F *pDat);
void	adjlms32fFree(asptAdjlms32f *adjlms);

/* API to filter through and update an asptAdjlms32f filters */
DT32F	adjlms32fFilterUpdate(asptAdjlms32f *adjlms, DT32F ref, DT32F prim);
DT32F	adjlms32fFilterOnly  (asptAdjlms32f *adjlms, DT32F ref);

/* API to retrieve the properties of an asptAdjlms32f filter */
DT32S	adjlms32fGetCoef(asptAdjlms32f *adjlms, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	adjlms32fGetDelayLine(asptAdjlms32f *adjlms, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
#define	adjlms32fGetCoefPtr(adjlms)				((adjlms)->coef)
#define	adjlms32fGetSecPathPtr(adjlms)			((adjlms)->secPath)
#define	adjlms32fGetEstSecPathPtr(adjlms)		((adjlms)->secPathEst)
#define	adjlms32fGetFilterLength(adjlms)		((adjlms)->filterLength)
#define	adjlms32fGetSecPathLength(adjlms)		((adjlms)->secPathLength)
#define	adjlms32fGetEstSecPathLength(adjlms)	((adjlms)->estSecPathLength)
#define	adjlms32fGetLastSensorSample(adjlms)	((adjlms)->sensor)
#define	adjlms32fGetStepSize(adjlms)			((adjlms)->stepSize)


/* API to set the properties of an asptAdjlms32f filter */
DT32S	adjlms32fReset(asptAdjlms32f *adjlms);
DT32S	adjlms32fResetDelayLine(asptAdjlms32f *adjlms);
DT32S	adjlms32fResize(asptAdjlms32f *adjlms, DT32S newLength);
DT32S	adjlms32fSetCoef(asptAdjlms32f *adjlms, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	adjlms32fSetDelayLine(asptAdjlms32f *adjlms, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	adjlms32fSetStepSize(adjlms, newStep)			((adjlms)->stepSize = newStep)
#define	adjlms32fSetSecPathPtr(adjlms, newPtr)			((adjlms)->secPath = newPtr)
#define	adjlms32fSetEstSecPathPtr(adjlms, newPtr)		((adjlms)->secPathEst = newPtr)
#define	adjlms32fSetSecPathLength(adjlms, newLength)	((adjlms)->secPathLength = newLength)
#define	adjlms32fSetEstSecPathLength(adjlms, newLength)	((adjlms)->estSecPathLength = newLength)

#ifdef __cplusplus
}
#endif

#endif /* ASPTADJLMS_H_ */
