/*  ********************************************************************
    ASPTBFDAF.C 
    Declares the Block Frequency Domain Adaptive Filter (BFDAF).
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef BFDAF_H_
#define BFDAF_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon32f.h"
#include "asptfft32f.h"

typedef struct _asptBfdaf32f {
	DT32S	filterLength;			/* filter length in samples */
	DT32S	blockLength;			/* new samples per block    */
	DT32S	fftLength;				/* fft length               */	
	DT32S	BO2P1;					/* length of complex vector */
	DT32S	constrain;				/* constrained BFDAF        */
	DT32S	C1;						/* fftLength - blockLength  */
	DT32S	C2;						/* fftLength - filterLength */
	DT32F	B2;						/* power smoothing pole     */

	DT32F	stepSize;				/* BFDAF step size          */
	DT32F	scale;					/* scaling factor           */
	DT32F	*xTime;					/* input vec. in T-domain   */
	DT32F	*xFreq;					/* input vec. in F-domain   */
	DT32F	*wFreq;					/* coef. vector F-domain	*/
	DT32F	*yFreq;					/* filter output            */
	DT32F	*pFreq;					/* input signal power       */
	DT32F	*pNew;					/* new input power          */
	DT32F	*w;						/* FFT twiddle factor table */
	DT32S	*ip;					/* work buffer for FFT      */
} asptBfdaf32f;

/* API to create and delete asptBfdaf32f filters */
DT32S	bfdaf32fInit(asptBfdaf32f *bfdaf,DT32S M,DT32S L,DT32F mu,DT32S cs,DT32S *ip,DT32F *w);
DT32S	bfdaf32fInitStatic(asptBfdaf32f *bfdaf,DT32S M,DT32S L,DT32F mu,DT32S cs,DT32F *pCof,DT32F *pDat,DT32S *ip,DT32F *w);
void	bfdaf32fFree(asptBfdaf32f *bfdaf);


/* API to filter and update asptBfdaf32f filters */
DT32S bfdaf32fFilterUpdate(asptBfdaf32f *bfdaf, DT32F *inp, DT32F *des, DT32F *out, DT32F *err);
DT32S bfdaf32fFilterOnly(asptBfdaf32f *bfdaf, DT32F *inp, DT32F *out);

/* API to retrieve the properties of asptBfdaf32f filters */
DT32S	bfdaf32fGetCoef(asptBfdaf32f *bfdaf, DT32F *dstBuf, DT32S ind, DT32S N, DT32S flip);
#define	bfdaf32fGetCoefPtr(bfdaf) 			((bfdaf)->wFreq)
#define	bfdaf32fGetFilterLength(bfdaf) 		((bfdaf)->filterLength)
#define	bfdaf32fGetBlockLength(bfdaf) 		((bfdaf)->blockLength)
#define	bfdaf32fGetFftLength(bfdaf) 		((bfdaf)->fftLength)
#define	bfdaf32fGetStepSize(bfdaf) 			((bfdaf)->stepSize)
#define	bfdaf32fGetConstrain(bfdaf) 		((bfdaf)->constrain)

/* API to set the properties of asptBfdaf32f filters */
DT32S	bfdaf32fReset(asptBfdaf32f *bfdaf);
DT32S	bfdaf32fResetDelayLine(asptBfdaf32f *bfdaf);
DT32S	bfdaf32fResize(asptBfdaf32f *bfdaf, DT32S newLen, DT32S newBlock);
DT32S	bfdaf32fSetCoef(asptBfdaf32f *bfdaf, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	bfdaf32fSetStepSize(bfdaf, newSs) 	((bfdaf)->stepSize = newSs)
#define	bfdaf32fSetConstrain(bfdaf, newC) 	((bfdaf)->constrain = newC)


#ifdef __cplusplus
}
#endif

#endif /* BFDAF_H_ */
