/*  ********************************************************************
    ASPTBLMS.H 

    Declares the Block Least Mean Squares (BLMS) adaptive filter algorithm.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTBLMS_H_
#define ASPTBLMS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon32f.h"

typedef struct _asptBlms32f {
	DT32S	filterLength;			/* filter length in samples		*/
	DT32S	blockLength;			/* new samples per block		*/
	DT32F	stepSize;				/* LMS step size				*/
	DT32F	*coef;					/* filter coef. vector			*/
	DT32F	*delayLine;				/* pointer to the filter data	*/
	DT32S	xIndex;
} asptBlms32f;

/* API to create and delete an asptBlms32f filters */
DT32S	blms32fInit (asptBlms32f *blms,DT32S L, DT32S B, DT32F mu);
DT32S	blms32fInitStatic(asptBlms32f *blms,DT32S L, DT32S B, DT32F mu, DT32F *pCof, DT32F *pDat);
void	blms32fFree(asptBlms32f *blms);

/* API to filter through and update an asptBlms32f filters */
DT32S	blms32fFilterUpdate        (asptBlms32f *blms, DT32F *inp, DT32F *des, DT32F *out, DT32F *err);
DT32S	blms32fFilterUpdateSignErr (asptBlms32f *blms, DT32F *inp, DT32F *des, DT32F *out, DT32F *err);
DT32S	blms32fFilterUpdateSignReg (asptBlms32f *blms, DT32F *inp, DT32F *des, DT32F *out, DT32F *err);
DT32S	blms32fFilterUpdateSignSign(asptBlms32f *blms, DT32F *inp, DT32F *des, DT32F *out, DT32F *err);
DT32S	blms32fFilterOnly  (asptBlms32f *blms, DT32F *inp, DT32F *out);

/* API to retrieve the properties of an asptBlms32f filter */
DT32S	blms32fGetCoef(asptBlms32f *blms, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	blms32fGetDelayLine(asptBlms32f *blms, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
#define	blms32fGetCoefPtr(blms)			((blms)->coef)
#define	blms32fGetFilterLength(blms)	((blms)->filterLength)
#define	blms32fGetStepSize(blms)		((blms)->stepSize)
#define	blms32fGetBlockLength(blms)		((blms)->blockLength)

/* API to set the properties of an asptBlms32f filter */
DT32S	blms32fReset(asptBlms32f *blms);
DT32S	blms32fResetDelayLine(asptBlms32f *blms);
DT32S	blms32fResize(asptBlms32f *blms, DT32S newL, DT32S newB);
DT32S	blms32fSetCoef(asptBlms32f *blms, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	blms32fSetDelayLine(asptBlms32f *blms, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	blms32fSetStepSize(blms, newStep)	((blms)->stepSize = newStep)
#define	blms32fSetBlockLength(blms, newBlock) ((blms)->blockLength = newBlock)

#ifdef __cplusplus
}
#endif

#endif /* ASPTBLMS_H_ */
