/*  ********************************************************************
    ASPTBNLMS.H 

    Declares the Block Normalized Least Mean Squares (BNLMS) adaptive 
	filter algorithm.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTBNLMS_H_
#define ASPTBNLMS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon32f.h"

typedef struct _asptBnlms32f {
	DT32S	filterLength;			/* filter length in samples		*/
	DT32S	blockLength;			/* new samples per block		*/
	DT32F	stepSize;				/* LMS step size				*/
	DT32F	*coef;					/* filter coef. vector			*/
	DT32F	power;					/* input signal power			*/
	DT32F	*delayLine;				/* pointer to the filter data	*/
	DT32S	xIndex;
} asptBnlms32f;

/* API to create and delete an asptBnlms32f filters */
DT32S	bnlms32fInit (asptBnlms32f *bnlms,DT32S L, DT32S B, DT32F mu);
DT32S	bnlms32fInitStatic(asptBnlms32f *bnlms,DT32S L, DT32S B, DT32F mu, DT32F *pCof, DT32F *pDat);
void	bnlms32fFree(asptBnlms32f *bnlms);

/* API to filter through and update an asptBnlms32f filters */
DT32S	bnlms32fFilterUpdate        (asptBnlms32f *bnlms, DT32F *inp, DT32F *des, DT32F *out, DT32F *err);
DT32S	bnlms32fFilterOnly  (asptBnlms32f *bnlms, DT32F *inp, DT32F *out);

/* API to retrieve the properties of an asptBnlms32f filter */
DT32S	bnlms32fGetCoef(asptBnlms32f *bnlms, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	bnlms32fGetDelayLine(asptBnlms32f *bnlms, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
#define	bnlms32fGetCoefPtr(bnlms)			((bnlms)->coef)
#define	bnlms32fGetFilterLength(bnlms)	((bnlms)->filterLength)
#define	bnlms32fGetStepSize(bnlms)		((bnlms)->stepSize)
#define	bnlms32fGetBlockLength(bnlms)		((bnlms)->blockLength)

/* API to set the properties of an asptBnlms32f filter */
DT32S	bnlms32fReset(asptBnlms32f *bnlms);
DT32S	bnlms32fResetDelayLine(asptBnlms32f *bnlms);
DT32S	bnlms32fResize(asptBnlms32f *bnlms, DT32S newL, DT32S newB);
DT32S	bnlms32fSetCoef(asptBnlms32f *bnlms, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	bnlms32fSetDelayLine(asptBnlms32f *bnlms, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	bnlms32fSetStepSize(bnlms, newStep)	((bnlms)->stepSize = newStep)
#define	bnlms32fSetBlockLength(bnlms, newBlock) ((bnlms)->blockLength = newBlock)

#ifdef __cplusplus
}
#endif

#endif /* ASPTBNLMS_H_ */
