/*  ********************************************************************
    COMMON.H 

    declares functions common to more than one algorithm in the package.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef COMMON_H_
#define COMMON_H_

#include <math.h>
#include <stdlib.h>
#include "asptdefs.h"

#ifdef __cplusplus
extern "C" {
#endif


/* Length calculation functions */
DT32S	nextPow2(const DT64S N);
DT32S	rsum(DT32S x, DT32S y);

/* DT32F vector operations */
DT32F	fir32f(DT32F s, DT32F x[], const DT32F h[], DT32S xLen, DT32S hLen, DT32S *pos);
DT32F	firAdj32f(DT32F s, DT32F x[], const DT32F h[], DT32S xLen, DT32S hLen, DT32S *pos);
DT32F	dotProdCircBuf32f(DT32F x[], const DT32F h[], DT32S xLen, DT32S hLen, DT32S pos);
DT32F	dotProdLinBuf32f(const DT32F x[], const DT32F h[], DT32S N);
DT32S	sum32f(DT32F *src,DT32F *sum, DT32S N);
DT32S	sumAbs32f(DT32F *src,DT32F *sum, DT32S N);
DT32S	copy32f(const DT32F *src, DT32F *dst, DT32S N);
DT32S	set32f(DT32F g, DT32F *srcdst, DT32S N);
DT32S	reset32f(DT32F *srcdst, DT32S N);
DT32S	scaleAdd32f(DT32F scale, const DT32F *src1, const DT32F *src2, DT32F *dst, DT32S N);
DT32S	scaleAddIp32f(DT32F scale, const DT32F *src, DT32F *srcdst, DT32S N);
DT32S	smooth32f(DT32F a1, DT32F *src1, DT32F a2, DT32F *srcdst, DT32S N);
DT32S	add32f(const DT32F *src1, const DT32F *src2, DT32F *dst, DT32S N);
DT32S	addIp32f(const DT32F *src, DT32F *srcdst, DT32S N);
DT32S	sub32f(const DT32F *src1, const DT32F *src2, DT32F *dst, DT32S N);
DT32S	scaleIp32f(DT32F scale, DT32F *srcdst, DT32S N);
DT32S	divIp32f(const DT32F *src, DT32F d, DT32F *srcdst, DT32S N);
DT32S	div32f(const DT32F *src1, const DT32F *src2, DT32F d, DT32F *dst, DT32S N);
DT32S	insertSampleLinBuf32f(DT32F x, DT32F *buf, DT32S N);

/* Complex DT32FC vector operations */
DT32S	powerSpectr32fc(const DT32FC *src, DT32F *dst, DT32S N);
DT32S	powerSpectrAdd32fc(const DT32FC *src, DT32F *srcdst, DT32S N);
DT32S	conjIp32fc(DT32FC *srcdst, DT32S N);
DT32S	conj32fc(DT32FC *src, DT32FC *dst, DT32S N);
DT32S	mpy32fc(const DT32FC *src1, const DT32FC *src2, DT32FC *dst, DT32S N);
DT32S	mpyIp32fc(const DT32FC *src, DT32FC *srcdst, DT32S N);
DT32S	mpyIp32f32fc(const DT32F *src, DT32FC *srcdst, DT32S N);
DT32S	mpyAdd32fc(const DT32FC *src1, const DT32FC *src2, DT32FC *srcdst, DT32S N);
DT32S	mpySub32fc(const DT32FC *src1, const DT32FC *src2, DT32FC *srcdst, DT32S N);
DT32S	divIp32f32fc(const DT32F *src, DT32F d, DT32FC *srcdst, DT32S N);
DT32S	subIp32fc(const DT32FC *src, DT32FC *srcdst, DT32S N);
DT32S	scaleIp32fc(DT32F scale, DT32FC *srcdst, DT32S N);

/* Half complex DT32FC vector operations */
DT32S	powerSpectrR32fc(const DT32FC *src, DT32F *dst, DT32S N);
DT32S	powerSpectrAddR32fc(const DT32FC *src, DT32F *srcdst, DT32S N);
DT32S	conjIpR32fc(DT32FC *srcdst, DT32S N);
DT32S	conjR32fc(DT32FC *src, DT32FC *dst, DT32S N);
DT32S	mpyR32fc(const DT32FC *src1, const DT32FC *src2, DT32FC *dst, DT32S N);
DT32S	mpyIpR32fc(const DT32FC *src, DT32FC *srcdst, DT32S N);
DT32S	mpyIp32fR32fc(const DT32F *src, DT32FC *srcdst, DT32S N);
DT32S	mpyAddR32fc(const DT32FC *src1, const DT32FC *src2, DT32FC *srcdst, DT32S N);
DT32S	mpySubR32fc(const DT32FC *src1, const DT32FC *src2, DT32FC *srcdst, DT32S N);
DT32S	divIp32fR32fc(const DT32F *src, DT32F d, DT32FC *srcdst, DT32S N);
DT32S	subIpR32fc(const DT32FC *src, DT32FC *srcdst, DT32S N);
DT32S	scaleIpR32fc(DT32F scale, DT32FC *srcdst, DT32S N);


#ifdef __cplusplus
}
#endif

#endif /* COMMON_H_ */


