/*  ********************************************************************
    DEFS.H 
	ASPT Type definitions
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef DEFS_H_
#define DEFS_H_

#ifdef __cplusplus
extern "C" {
#endif

#if defined( _WIN32 ) || defined ( _WIN64 ) 
  #define __STDCALL  __stdcall
  #define __CDECL    __cdecl
  #define __INT64    __int64
  #define __UINT64    unsigned __int64
#else
  #define __STDCALL
  #define __CDECL
/* #ifdef __GNUC__
  #define __INT64    long long int
  #define __UINT64    unsigned long long int 
#else
*/
  #define __INT64    long int
  #define __UINT64    unsigned long int
#endif 

/*#endif
 */

#ifdef FREE
#define MAX_FIR_LEN 32
#define MAX_LAT_LEN 10
#define MAX_IIR_LEN 10
#endif

typedef unsigned char  DT8U;
typedef unsigned short DT16U;
typedef unsigned int   DT32U;
typedef signed char    DT8S;
typedef signed short   DT16S;
typedef signed int     DT32S;
typedef float          DT32F;
#ifdef __GNUC__
typedef long  int  DT64S;
typedef unsigned long int      DT64U;
#else
typedef __INT64        DT64S;
typedef __UINT64       DT64U;
#endif
typedef double         DT64F;

typedef enum {
	dt8u,
	dt16u,
	dt32u,
	dt8s,
	dt16s,
	dt32s,
	dt32f,
	dt64s,
	dt64u,
	dt64f,
	dt8sc,
	dt16sc,
	dt32sc,
	dt64sc,
	dt32fc,
	dt64fc
}dataType;
/*  ********************************************************************
							 Lib version 
    ******************************************************************** */

#define asptVersion (2.1f)

#define MAX( a, b ) ( ((a) > (b)) ? (a) : (b) )
#define MIN( a, b ) ( ((a) < (b)) ? (a) : (b) )
#define ABS(a)      ( (a)  >  0   ? (a) :-(a) )
/*#define SGN(a)      ( (a) ==  0   ?  0 : ((a) < 0 ? (-1): (1)) )*/
#define SGN(a)      ( (a) < 0 ? (-1): (1) )

/*  ********************************************************************
							 Complex Numbers 
    ******************************************************************** */

typedef struct {
    DT8S  re;
    DT8S  im;
} DT8SC;

typedef struct {
    DT16S  re;
    DT16S  im;
} DT16SC;

typedef struct {
    DT32S  re;
    DT32S  im;
} DT32SC;

typedef struct {
    DT32F  re;
    DT32F  im;
} DT32FC;

typedef struct {
    DT64S  re;
    DT64S  im;
} DT64SC;

typedef struct {
    DT64F  re;
    DT64F  im;
} DT64FC;


#define cplx(r,i,c) (c).re=r; (c).im=i
#define Real(x) ((x).re)
#define Imag(x) ((x).im)
#define Conj(x) (x).im = -Imag((x))
#define Conj2(x,y) (y).re = Real(x); y.im = -Imag(x)
#define cplxZero(a) (a).re=0; (a).im=0
#define cplxSet(x,a,b) (x).re=a; (x).im=b

/*  ********************************************************************
						 Complex - Complex Operators 
    ******************************************************************** */

/* square magnitude of a complex number a */
#define cplxMag2(a) ((a).re*(a).re + (a).im*(a).im)

/* c = a + b ; a,b,c are complex */
#define cplxAdd(a,b,c) (c).re=(a).re+(b).re;(c).im=(a).im+(b).im

/* c = a - b ;  a,b,c are complex */
#define cplxSub(a,b,c) (c).re=(a).re-(b).re;(c).im=(a).im-(b).im

/* c = a * b ; a,b,c are complex */
#define cplxMpy(a,b,c) (c).re=(a).re*(b).re -(a).im*(b).im;(c).im=(a).re*(b).im+(a).im*(b).re

/* c = a / b ; a,b,c are complex ; d=cmplxMag2(b)*/
#define cplxDiv(a,b,c,d) d=cplxMag2(b); Conj(b);cplxMpy(a,b,c); ((c).re) /= d; ((c).im) /= d; Conj(b)

/* c += a * b ; a,b,c are complex */
#define cplxMpyAdd(a,b,c) (c).re+=(a).re*(b).re -(a).im*(b).im;(c).im+=(a).re*(b).im+(a).im*(b).re

/* c -= a * b ; a,b,c are complex */
#define cplxMpySb(a,b,c) (c).re-=(a).re*(b).re -(a).im*(b).im;(c).im-=(a).re*(b).im+(a).im*(b).re

/*  ********************************************************************
						 omplex - Scalar Operators 
    ******************************************************************** */

/* c = b + a ; b,c are complex ; a is scalar */
#define cplxAddS(a,b,c) (c).re=(b).re + (a);(c).im=(b).im

/* c = b - a ;  b,c are complex ; a is scalar */
#define cplxSubS(a,b,c) (c).re=(b).re-(a);(c).im=(b).im

/* c = a * b ; b,c are complex ; a is scalar  */
#define cplxMpyS(a,b,c) (c).re=(b).re*(a);(c).im=(b).im*(a)

/* c = b / a ; b,c are complex ; a is scalar */
#define cplxDivS(a,b,c) (c).re=(b).re/a;(c).im=(b).im/(a)

/* c = a - b ;  b,c are complex ; a is scalar */
#define cplxSSub(a,b,c) (c).re=a-(b).re;(c).im=-(b).im

/* c = a / b ; b,c are complex ; a is scalar*/
#define cplxSDiv(a,b,c,d) d=cplxMag2(b); Conj(b);(c).re=a*(b).re/d; (c).im=a*(b).im/d; Conj(b)

/*  ********************************************************************
						 ASPT Constants 
    ******************************************************************** */

#define PI    ( 3.14159265358979323846 )  /* ANSI C does not support M_PI */
#define PIX2  ( 6.28318530717958647692 )  /* 2*pi                         */
#define PIO2  ( 1.57079632679489661923 )  /* pi/2                         */
#define PI4   ( 0.78539816339744830961 )  /* pi/4                         */
#define PI180 ( 0.01745329251994329577 )  /* pi/180                       */
#define RPI   ( 0.31830988618379067154 )  /* 1/pi                         */
#define SQRT2 ( 1.41421356237309504880 )  /* sqrt(2)                      */
#define SQRT3 ( 1.73205080756887729353 )  /* sqrt(3)                      */
#define LN2   ( 0.69314718055994530942 )  /* ln(2)                        */
#define LN3   ( 1.09861228866810969139 )  /* ln(3)                        */
#define E     ( 2.71828182845904523536 )  /* e                            */
#define RE    ( 0.36787944117144232159 )  /* 1/e                          */
#define EPS23 ( 1.19209289e-07f )
#define EPS52 ( 2.2204460492503131e-016 )

#define MAX_8U     ( 0xFF)
#define MAX_16U    ( 0xFFFF)
#define MAX_32U    ( 0xFFFFFFFF)
#define MIN_8S     (-128)
#define MAX_8S     ( 127)
#define MIN_16S    (-32768)
#define MAX_16S    ( 32767)
#define MIN_32S    (-2147483647-1)
#define MAX_32S    ( 2147483647 )

#if defined( _WIN32 ) || defined ( _WIN64 )
  #define MAX_64S  ( 9223372036854775807i64 )
  #define MIN_64S  (-9223372036854775807i64 - 1 )
#else
  #define MAX_64S  ( 9223372036854775807LL )
  #define MIN_64S  (-9223372036854775807LL - 1 )
#endif
/*  ********************************************************************
						 ASPT Error Codes 
    ******************************************************************** */
#define ASPT_NO_ERR         0
#define ASPT_NULL_PTR_ERR   -1
#define ASPT_RANGE_ERR      -2
#define ASPT_NO_MEM_ERR     -3
#define ASPT_UNKN_ERR       -4

/*  ********************************************************************
						 Function prototypes 
    ******************************************************************** */
DT32F asptGetVersion(void);
DT32S asptInitStatic(DT64S code);


#ifdef __cplusplus
}
#endif

#endif /* DEFS_H_ */
