/*  ********************************************************************
    FIR.H 

    Declares a Finite Impulse Response filter.

    This version uses a circular buffer of length equals to the filter 
	length.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */

#ifndef FIR_H_
#define FIR_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon32f.h"

typedef struct _asptFir32f {
	DT32F	*coef;			/* coefficient vector	      */
	DT32S	filterLength;	/* coef vector length	      */
	DT32F	*delayLine;		/* circular buffer		      */
	DT32S	idx;			/* index to the oldest sample */
} asptFir32f;

/* API to create and delete asptFir32f filters */
DT32S	fir32fInit (asptFir32f *fir, DT32S L, DT32F *pCof);
DT32S	fir32fInitStatic(asptFir32f *fir, DT32S L, DT32F *pCof, DT32F *pDat);
void	fir32fFree(asptFir32f *fir);

/* API to filter through asptFir32f filters */
DT32F	fir32fFilter(asptFir32f *fir, DT32F inSample);
DT32F	fir32fLCFilter(asptFir32f *fir);
DT32S	fir32fBlockFilter(asptFir32f *fir, DT32F *in, DT32F *out,DT32S BlockLength);

/* API to retrieve the properties of an asptFir32f filter */
DT32S	fir32fGetCoef(asptFir32f *fir, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	fir32fGetDelayLine(asptFir32f *fir, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
#define	fir32fGetCoefPtr(fir)			((fir)->coef)
#define	fir32fGetFilterLength(fir)		((fir)->filterLength)
#define	fir32fGetDelayLineLength(fir)	((fir)->filterLength)

/* API to set the properties of an asptFir32f filter */
DT32S	fir32fResetDelayLine(asptFir32f *fir);
DT32S	fir32fResize(asptFir32f *fir, DT32S newLength);
DT32S	fir32fSetCoef(asptFir32f *fir, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	fir32fSetDelayLine(asptFir32f *fir, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);

#ifdef __cplusplus
}
#endif

#endif /* FIR_H_ */
