/*  ********************************************************************
    ASPTFXLMS.H 

    Declares the Filtered-X LMS algorithm.
    This version is intended for simulation purposes and matches 
    the Matlab asptfxlms and init_fxlms functions.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTFXLMS_H_
#define ASPTFXLMS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon32f.h"

typedef struct _asptFxlms32f {
	DT32S	filterLength;			/* filter length in samples		*/
	DT32S	secPathLength;			/* secondary path length		*/
	DT32S	estSecPathLength;		/* estimated sec. path length	*/
	DT32S	refLength;				/* input signal buffer length   */
	DT32F	stepSize;				/* LMS step size				*/
	DT32F	sensor;					/* last sensor sample			*/
	DT32F	power;                  /* last estimated input power	*/
	DT32F	*coef;					/* filter coef. vector			*/
	DT32F	*secPath;				/* secondary path				*/
	DT32F	*secPathEst;			/* estimated secondary path		*/
	DT32F	*yBuf;					/* buffer for output signal		*/
	DT32F	*fxBuf;					/* filtered-x signalbuffer		*/
	DT32F	*delayLine;				/* pointer to the filter data	*/
	DT32S	xIndex;
	DT32S	yIndex;
	DT32S	fIndex;
} asptFxlms32f;

/* API to create and delete an asptFxlms32f filters */
DT32S	fxlms32fInit (asptFxlms32f *fxlms,DT32S L, DT32F mu, DT32F *sPath, DT32S sL, DT32F *sPathEst, DT32S seL);
DT32S	fxlms32fInitStatic(asptFxlms32f *fxlms,DT32S L, DT32F mu, DT32F *sPath, DT32S sL, DT32F *sPathEst, DT32S seL, DT32F *pCof, DT32F *pDat);
void	fxlms32fFree(asptFxlms32f *fxlms);

/* API to filter through and update an asptFxlms32f filters */
DT32F	fxlms32fFilterUpdate(asptFxlms32f *fxlms, DT32F ref, DT32F prim);
DT32F	fxlms32fFilterOnly  (asptFxlms32f *fxlms, DT32F ref);

/* API to retrieve the properties of an asptFxlms32f filter */
DT32S	fxlms32fGetCoef(asptFxlms32f *fxlms, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	fxlms32fGetDelayLine(asptFxlms32f *fxlms, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
#define	fxlms32fGetCoefPtr(fxlms)			((fxlms)->coef)
#define	fxlms32fGetSecPathPtr(fxlms)		((fxlms)->secPath)
#define	fxlms32fGetEstSecPathPtr(fxlms)		((fxlms)->secPathEst)
#define	fxlms32fGetFilterLength(fxlms)		((fxlms)->filterLength)
#define	fxlms32fGetSecPathLength(fxlms)		((fxlms)->secPathLength)
#define	fxlms32fGetEstSecPathLength(fxlms)	((fxlms)->estSecPathLength)
#define	fxlms32fGetLastSensorSample(fxlms)	((fxlms)->sensor)
#define	fxlms32fGetStepSize(fxlms)			((fxlms)->stepSize)


/* API to set the properties of an asptFxlms32f filter */
DT32S	fxlms32fReset(asptFxlms32f *fxlms);
DT32S	fxlms32fResetDelayLine(asptFxlms32f *fxlms);
DT32S	fxlms32fResize(asptFxlms32f *fxlms, DT32S newLength);
DT32S	fxlms32fSetCoef(asptFxlms32f *fxlms, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	fxlms32fSetDelayLine(asptFxlms32f *fxlms, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	fxlms32fSetStepSize(fxlms, newStep)				((fxlms)->stepSize = newStep)
#define	fxlms32fSetSecPathPtr(fxlms, newPtr)			((fxlms)->secPath = newPtr)
#define	fxlms32fSetEstSecPathPtr(fxlms, newPtr)			((fxlms)->secPathEst = newPtr)
#define	fxlms32fSetSecPathLength(fxlms, newLength)		((fxlms)->secPathLength = newLength)
#define	fxlms32fSetEstSecPathLength(fxlms, newLength)	((fxlms)->estSecPathLength = newLength)

#ifdef __cplusplus
}
#endif

#endif /* ASPTFXLMS_H_ */
