/*  ********************************************************************
    ASPTLBPEF.H 
    Declares the Least Mean Square adaptive Lattice Backward Prediction 
	Error Filter (LBPEF).
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */

#ifndef ASPTLBPEF_H_
#define ASPTLBPEF_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon32f.h"

typedef struct _asptLbpef32f {
	DT32S	filterLength;	/* filter length in samples  */
	DT32F	pcStepSize;		/* predictor coef. step size  */
	DT32F	error;			/* last error sample */
	DT32S	firFlag;		/* flag for calculating FIR coef. */
	DT32S	index;			/* index of last inserted input sample */
	DT32F	*parcor;		/* PARCOR coef. vector */
	DT32F	*coef;			/* linear combiner coef. vector */
	DT32F	*bpe;			/* backward prediction error vector */
	DT32F	*bpePower;		/* power estimate of bpe */
	DT32F	*scratch;		/* working buffer     */
	DT32F	*inBuf;			/* input signal buffer */
           
} asptLbpef32f;

/* API to create and delete an asptLbpef32f filter */
DT32S	lbpef32fInit(asptLbpef32f *lbpef, DT32S L, DT32F mu_p);
DT32S	lbpef32fInitStatic(asptLbpef32f *lbpef,DT32S L, DT32F mu_p, DT32F *pCof, DT32F *pDat);
void	lbpef32fFree(asptLbpef32f *lbpef);

/* API to filter and update an asptLbpef32f filter */
DT32F	lbpef32fFilterUpdate(asptLbpef32f *lbpef, DT32F x);
DT32F	lbpef32fFilterOnly(asptLbpef32f *lbpef, DT32F x);

/* API to retrieve the properties of an asptLbpef32f filter */
DT32S	lbpef32fGetCoef(asptLbpef32f *lbpef, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	lbpef32fGetParcor(asptLbpef32f *lbpef, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	lbpef32fGetDelayLine(asptLbpef32f *lbpef, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
#define	lbpef32fGetCoefPtr(lbpef)			((lbpef)->coef)
#define	lbpef32fGetParcorPtr(lbpef)			((lbpef)->parcor)
#define	lbpef32fGetDelayLinePtr(lbpef)		((lbpef)->bpe)
#define	lbpef32fGetFilterLength(lbpef)		((lbpef)->filterLength)
#define	lbpef32fGetPcStepSize(lbpef)		((lbpef)->pcStepSize)
#define	lbpef32fGetLastErrorSample(lbpef)	((lbpef)->error)
#define	lbpef32fGetFirFlag(lbpef)			((lbpef)->firFlag)

/* API to set the properties of an asptLbpef32f filter */
DT32S	lbpef32fReset(asptLbpef32f *lbpef);
DT32S	lbpef32fResize(asptLbpef32f *lbpef, DT32S newLength);
DT32S	lbpef32fSetCoef(asptLbpef32f *lbpef, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	lbpef32fSetParcor(asptLbpef32f *lbpef, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	lbpef32fSetDelayLine(asptLbpef32f *lbpef, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	lbpef32fSetPcStepSize(lbpef, newStep) ((lbpef)->pcStepSize = newStep)
#define	lbpef32fSetFirFlag(lbpef, newFlag) ((lbpef)->firFlag = newFlag)

#ifdef __cplusplus
}
#endif
#endif /* ASPTLBPEF_H_ */

