/*  ********************************************************************
    ASPTLFPEF.H 
    Declares the Least Mean Square adaptive Lattice Forward Prediction 
	Error Filter (LFPEF).
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */

#ifndef ASPTLFPEF_H_
#define ASPTLFPEF_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon32f.h"

typedef struct _asptLfpef32f {
	DT32S	filterLength;	/* filter length in samples  */
	DT32F	pcStepSize;		/* predictor coef. step size  */
	DT32F	error;			/* last error sample */
	DT32S	firFlag;		/* flag for calculating FIR coef. */
	DT32S	index;			/* index of last inserted input sample */
	DT32F	*parcor;		/* PARCOR coef. vector */
	DT32F	*coef;			/* linear combiner coef. vector */
	DT32F	*bpe;			/* backward prediction error vector */
	DT32F	*bpePower;		/* power estimate of bpe */
	DT32F	*scratch;		/* working buffer                */
} asptLfpef32f;

/* API to create and delete an asptLfpef32f filter */
DT32S	lfpef32fInit(asptLfpef32f *lfpef, DT32S L, DT32F mu_p);
DT32S	lfpef32fInitStatic(asptLfpef32f *lfpef,DT32S L, DT32F mu_p, DT32F *pCof, DT32F *pDat);
void	lfpef32fFree(asptLfpef32f *lfpef);

/* API to filter and update an asptLfpef32f filter */
DT32F	lfpef32fFilterUpdate(asptLfpef32f *lfpef, DT32F x);
DT32F	lfpef32fFilterOnly(asptLfpef32f *lfpef, DT32F x);

/* API to retrieve the properties of an asptLfpef32f filter */
DT32S	lfpef32fGetCoef(asptLfpef32f *lfpef, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	lfpef32fGetParcor(asptLfpef32f *lfpef, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	lfpef32fGetDelayLine(asptLfpef32f *lfpef, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
#define	lfpef32fGetCoefPtr(lfpef)			((lfpef)->coef)
#define	lfpef32fGetParcorPtr(lfpef)			((lfpef)->parcor)
#define	lfpef32fGetDelayLinePtr(lfpef)		((lfpef)->bpe)
#define	lfpef32fGetFilterLength(lfpef)		((lfpef)->filterLength)
#define	lfpef32fGetPcStepSize(lfpef)		((lfpef)->pcStepSize)
#define	lfpef32fGetLastErrorSample(lfpef)	((lfpef)->error)
#define	lfpef32fGetFirFlag(lfpef)			((lfpef)->firFlag)

/* API to set the properties of an asptLfpef32f filter */
DT32S	lfpef32fReset(asptLfpef32f *lfpef);
DT32S	lfpef32fResize(asptLfpef32f *lfpef, DT32S newLength);
DT32S	lfpef32fSetCoef(asptLfpef32f *lfpef, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	lfpef32fSetParcor(asptLfpef32f *lfpef, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	lfpef32fSetDelayLine(asptLfpef32f *lfpef, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	lfpef32fSetPcStepSize(lfpef, newStep) ((lfpef)->pcStepSize = newStep)
#define	lfpef32fSetFirFlag(lfpef, newFlag) ((lfpef)->firFlag = newFlag)

#ifdef __cplusplus
}
#endif
#endif /* ASPTLFPEF_H_ */

