/*  ********************************************************************
    ASPTLMS.H 

    Declares the Least Mean Squares (LMS) adaptive filter algorithm.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTLMS_H_
#define ASPTLMS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon32f.h"

typedef struct _asptLms32f {
	DT32S	filterLength;			/* filter length in samples		*/
	DT32F	stepSize;				/* LMS step size				*/
	DT32F	error;					/* last error sample			*/
	DT32F	*coef;					/* filter coef. vector			*/
	DT32F	*delayLine;				/* pointer to the filter data	*/
	DT32S	xIndex;
} asptLms32f;

/* API to create and delete an asptLms32f filters */
DT32S	lms32fInit (asptLms32f *lms,DT32S L, DT32F mu);
DT32S	lms32fInitStatic(asptLms32f *lms,DT32S L, DT32F mu, DT32F *pCof, DT32F *pDat);
void	lms32fFree(asptLms32f *lms);

/* API to filter through and update an asptLms32f filters */
DT32F	lms32fFilterUpdate        (asptLms32f *lms, DT32F inp, DT32F des);
DT32F	lms32fFilterUpdateSignErr (asptLms32f *lms, DT32F inp, DT32F des);
DT32F	lms32fFilterUpdateSignReg (asptLms32f *lms, DT32F inp, DT32F des);
DT32F	lms32fFilterUpdateSignSign(asptLms32f *lms, DT32F inp, DT32F des);
DT32F	lms32fFilterOnly  (asptLms32f *lms, DT32F inp);

DT32F	lms32fLCFilterUpdate        (asptLms32f *lms, DT32F des);
DT32F	lms32fLCFilterUpdateSignErr (asptLms32f *lms, DT32F des);
DT32F	lms32fLCFilterUpdateSignReg (asptLms32f *lms, DT32F des);
DT32F	lms32fLCFilterUpdateSignSign(asptLms32f *lms, DT32F des);
DT32F	lms32fLCFilterOnly  (asptLms32f *lms);

/* API to retrieve the properties of an asptLms32f filter */
DT32S	lms32fGetCoef(asptLms32f *lms, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	lms32fGetDelayLine(asptLms32f *lms, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
#define	lms32fGetCoefPtr(lms)			((lms)->coef)
#define	lms32fGetFilterLength(lms)		((lms)->filterLength)
#define	lms32fGetDelayLineLength(lms)	((lms)->filterLength)
#define	lms32fGetLastErrorSample(lms)	((lms)->error)
#define	lms32fGetStepSize(lms)			((lms)->stepSize)


/* API to set the properties of an asptLms32f filter */
DT32S	lms32fReset(asptLms32f *lms);
DT32S	lms32fResetDelayLine(asptLms32f *lms);
DT32S	lms32fResize(asptLms32f *lms, DT32S newLength);
DT32S	lms32fSetCoef(asptLms32f *lms, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	lms32fSetDelayLine(asptLms32f *lms, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	lms32fSetStepSize(lms, newStep)	((lms)->stepSize = newStep)

#ifdef __cplusplus
}
#endif

#endif /* ASPTLMS_H_ */
