/*  ********************************************************************
    ASPTLMSLATTICE.H 
    Declares the LMS LATTICE Joint Process Estimator adaptive algorithm.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */

#ifndef ASPTLMSLATTICE_H_
#define ASPTLMSLATTICE_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon32f.h"

typedef struct _asptLmslattice32f {
	DT32S	filterLength;	/* filter length in samples  */
	DT32F	pcStepSize;		/* predictor coef. step size  */
	DT32F	lcStepSize;		/* linear combiner step size  */
	DT32F	error;			/* last error sample */
	DT32S	updateFlag;		/* PARCOR update flag */
	DT32F	*parcor;		/* PARCOR coef. vector */
	DT32F	*coef;			/* linear combiner coef. vector */
	DT32F	*bpe;			/* backward prediction error vector */
	DT32F	*bpePower;		/* power estimate of bpe */
	DT32F	*scratch;		/* working buffer            */
           
} asptLmslattice32f;

/* API to create and delete an asptLmslattice32f filter */
DT32S	lmslattice32fInit(asptLmslattice32f *lmsl, DT32S L, DT32F mu_p, DT32F mu_c);
DT32S	lmslattice32fInitStatic(asptLmslattice32f *lmsl,DT32S L, DT32F mu_p, DT32F mu_c, DT32F *pCof, DT32F *pDat);
void	lmslattice32fFree(asptLmslattice32f *lmsl);

/* API to filter and update an asptLmslattice32f filter */
DT32F	lmslattice32fFilterUpdate(asptLmslattice32f *lmsl, DT32F x, DT32F d);
DT32F	lmslattice32fFilterOnly(asptLmslattice32f *lmsl, DT32F x);

/* API to retrieve the properties of an asptLmslattice32f filter */
DT32S	lmslattice32fGetCoef(asptLmslattice32f *lmsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	lmslattice32fGetParcor(asptLmslattice32f *lmsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	lmslattice32fGetDelayLine(asptLmslattice32f *lmsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
#define	lmslattice32fGetCoefPtr(lmsl)			((lmsl)->coef)
#define	lmslattice32fGetParcorPtr(lmsl)			((lmsl)->parcor)
#define	lmslattice32fGetDelayLinePtr(lmsl)		((lmsl)->bpe)
#define	lmslattice32fGetFilterLength(lmsl)		((lmsl)->filterLength)
#define	lmslattice32fGetLcStepSize(lmsl)		((lmsl)->lcStepSize)
#define	lmslattice32fGetPcStepSize(lmsl)		((lmsl)->pcStepSize)
#define	lmslattice32fGetLastErrorSample(lmsl)	((lmsl)->error)
#define	lmslattice32fGetParcorUpdateFlag(lmsl)	((lmsl)->updateFlag)

/* API to set the properties of an asptLmslattice32f filter */
DT32S	lmslattice32fReset(asptLmslattice32f *lmsl);
DT32S	lmslattice32fResize(asptLmslattice32f *lmsl, DT32S newLength);
DT32S	lmslattice32fSetCoef(asptLmslattice32f *lmsl, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	lmslattice32fSetParcor(asptLmslattice32f *lmsl, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	lmslattice32fSetDelayLine(asptLmslattice32f *lmsl, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	lmslattice32fSetLcStepSize(lmsl, newStep) ((lmsl)->lcStepSize = newStep)
#define	lmslattice32fSetPcStepSize(lmsl, newStep) ((lmsl)->pcStepSize = newStep)
#define	lmslattice32fSetParcorUpdateFlag(lmsl, newFlag) ((lmsl)->updateFlag = newFlag)

#ifdef __cplusplus
}
#endif
#endif /* ASPTLMSLATTICE_H_ */

