/*  ********************************************************************
    ASPTNLMS.H 

    Declares the Normalized LMS adaptive filter algorithm.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTNLMS_H_
#define ASPTNLMS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon32f.h"

typedef struct _asptNlms32f {
	DT32S	filterLength;			/* filter length in samples		*/
	DT32F	stepSize;				/* LMS step size				*/
	DT32F	error;					/* last error sample			*/
	DT32F	power;                  /* last estimated input power	*/
	DT32F	*coef;					/* filter coef. vector			*/
	DT32F	*delayLine;				/* pointer to the filter data	*/
	DT32S	xIndex;
} asptNlms32f;

/* API to create and delete an asptNlms32f filters */
DT32S	nlms32fInit (asptNlms32f *nlms,DT32S L, DT32F mu);
DT32S	nlms32fInitStatic(asptNlms32f *nlms,DT32S L, DT32F mu, DT32F *pCof, DT32F *pDat);
void	nlms32fFree(asptNlms32f *nlms);

/* API to filter through and update an asptNlms32f filters */
DT32F	nlms32fFilterUpdate(asptNlms32f *nlms, DT32F inp, DT32F des);
DT32F	nlms32fFilterOnly  (asptNlms32f *nlms, DT32F inp);
DT32F	nlms32fLCFilterUpdate(asptNlms32f *nlms, DT32F des);
DT32F	nlms32fLCFilterOnly  (asptNlms32f *nlms);

/* API to retrieve the properties of an asptNlms32f filter */
DT32S	nlms32fGetCoef(asptNlms32f *nlms, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	nlms32fGetDelayLine(asptNlms32f *nlms, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
#define	nlms32fGetCoefPtr(nlms)			((nlms)->coef)
#define	nlms32fGetFilterLength(nlms)	((nlms)->filterLength)
#define	nlms32fGetDelayLineLength(nlms)	((nlms)->filterLength)
#define	nlms32fGetLastErrorSample(nlms)	((nlms)->error)
#define	nlms32fGetStepSize(nlms)		((nlms)->stepSize)


/* API to set the properties of an asptNlms32f filter */
DT32S	nlms32fReset(asptNlms32f *nlms);
DT32S	nlms32fResetDelayLine(asptNlms32f *nlms);
DT32S	nlms32fResize(asptNlms32f *nlms, DT32S newLength);
DT32S	nlms32fSetCoef(asptNlms32f *nlms, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	nlms32fSetDelayLine(asptNlms32f *nlms, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	nlms32fSetStepSize(nlms, newStep)	((nlms)->stepSize = newStep)

#ifdef __cplusplus
}
#endif

#endif /* ASPTNLMS_H_ */
