/*  ********************************************************************
    ASPTPBFDAF.C 
    Declares the Partitioned Block Frequency Domain Adaptive Filter (PBFDAF).
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef PBFDAF_H_
#define PBFDAF_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon32f.h"
#include "asptfft32f.h"

typedef struct _asptPbfdaf32f {
	DT32S	filterLength;			/* filter length in samples */
	DT32S	blockLength;			/* new samples per block    */
	DT32S	partitions;				/* number of partitions		*/
	DT32S	fftLength;				/* fft length               */	
	DT32S	matrixSize;				/* data matrix size			*/
	DT32S	BO2P1;					/* length of complex vector */
	DT32S	constrain;				/* constrained BFDAF        */
	DT32S	C1;						/* fftLength - blockLength  */
	DT32S	C2;						/* fftLength - filterLength */
	DT32F	B2;						/* power smoothing pole     */
	DT32S	curIndex;				/* index variable			*/
	DT32F	stepSize;				/* BFDAF step size          */
	DT32F	scale;					/* scaling factor           */
	DT32F	*xTime;					/* input vec. in T-domain   */
	DT32F	*xFreq;					/* input vec. in F-domain   */
	DT32F	*wFreq;					/* coef. vector F-domain	*/
	DT32F	*yFreq;					/* filter output            */
	DT32F	*pFreq;					/* input signal power       */
	DT32F	*pNew;					/* new input power          */
	DT32F	*w;						/* FFT twiddle factor table */
	DT32S	*ip;					/* work buffer for FFT      */
} asptPbfdaf32f;

/* API to create and delete asptPbfdaf32f filters */
DT32S	pbfdaf32fInit(asptPbfdaf32f *pbfdaf,DT32S M,DT32S L,DT32S N,DT32F mu,DT32S cs,DT32S *ip,DT32F *w);
DT32S	pbfdaf32fInitStatic(asptPbfdaf32f *pbfdaf,DT32S M,DT32S L,DT32S N,DT32F mu,DT32S cs,DT32F *pCof,DT32F *pDat,DT32S *ip,DT32F *w);
void	pbfdaf32fFree(asptPbfdaf32f *pbfdaf);


/* API to filter and update asptPbfdaf32f filters */
DT32S pbfdaf32fFilterUpdate(asptPbfdaf32f *pbfdaf, DT32F *inp, DT32F *des, DT32F *out, DT32F *err);
DT32S pbfdaf32fFilterOnly(asptPbfdaf32f *pbfdaf, DT32F *inp, DT32F *out);

/* API to retrieve the properties of asptPbfdaf32f filters */
DT32S	pbfdaf32fGetCoef(asptPbfdaf32f *pbfdaf, DT32F *dstBuf, DT32S ind, DT32S N, DT32S flip);
#define	pbfdaf32fGetCoefPtr(pbfdaf) 		((pbfdaf)->wFreq)
#define	pbfdaf32fGetFilterLength(pbfdaf) 	((pbfdaf)->filterLength)
#define	pbfdaf32fGetBlockLength(pbfdaf) 	((pbfdaf)->blockLength)
#define	pbfdaf32fGetFftLength(pbfdaf) 		((pbfdaf)->fftLength)
#define	pbfdaf32fGetStepSize(pbfdaf) 		((pbfdaf)->stepSize)
#define	pbfdaf32fGetConstrain(pbfdaf) 		((pbfdaf)->constrain)
#define	pbfdaf32fGetPartitions(pbfdaf)		((pbfdaf)->partitions)

/* API to set the properties of asptPbfdaf32f filters */
DT32S	pbfdaf32fReset(asptPbfdaf32f *pbfdaf);
DT32S	pbfdaf32fResetDelayLine(asptPbfdaf32f *pbfdaf);
DT32S	pbfdaf32fResize(asptPbfdaf32f *pbfdaf, DT32S newLen, DT32S newBlock, DT32S newPart);
DT32S	pbfdaf32fSetCoef(asptPbfdaf32f *pbfdaf, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	pbfdaf32fSetStepSize(pbfdaf, newSs) 	((pbfdaf)->stepSize = newSs)
#define	pbfdaf32fSetConstrain(pbfdaf, newC) 	((pbfdaf)->constrain = newC)


#ifdef __cplusplus
}
#endif

#endif /* PBFDAF_H_ */
