/*  ********************************************************************
    ASPTRLS.H 

    Declares the Recursive Least Squares (RLS) adaptive filter algorithm.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */

#ifndef ASPTRLS_H_
#define ASPTRLS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon32f.h"

#define ZUINIG

typedef struct _asptRls32f {
	DT32S		filterLength;	/* filter length in samples */
#ifndef ZUINIG
	DT32S		lenSquare;		/* filterLength ^ 2         */
#endif
	DT32F	forgetFactor;	/* forgetting factor        */
	DT32F	recForgetFactor;/* 1/(forgetFactor)         */
	DT32F	error;			/* last error sample		*/
	DT32F	*coef;			/* filter coef. vector		*/
	DT32F	*delayLine;		/* delay line vector		*/
	DT32F	*acor;			/* auto-corr. matrix inverse*/
	DT32F	*scratch;		/* working buffer           */

} asptRls32f;

/* API to create and delete an asptRls32f filters */
DT32S	rls32fInit (asptRls32f *rls, DT32S L, DT32F mu, DT32F b);
DT32S	rls32fInitStatic(asptRls32f *rls, DT32S L, DT32F mu, DT32F b, DT32F *pCof, DT32F *pDat);
void	rls32fFree(asptRls32f *rls);

/* API to filter through and update an asptRls32f filters */
DT32F	rls32fFilterUpdate(asptRls32f *rls, DT32F inp, DT32F des);
DT32F	rls32fFilterOnly(asptRls32f *rls, DT32F inp);
DT32F	rls32fLCFilterUpdate(asptRls32f *rls, DT32F des);
DT32F	rls32fLCFilterOnly(asptRls32f *rls);

/* API to retrieve the properties of an asptRls32f objects */
DT32S	rls32fGetCoef(asptRls32f *rls, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rls32fGetDelayLine(asptRls32f *rls, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
#define	rls32fGetCoefPtr(rls)			((rls)->coef)
#define	rls32fGetFilterLength(rls)		((rls)->filterLength)
#define	rls32fGetDelayLineLength(rls)	((rls)->filterLength)
#define	rls32fGetLastErrorSample(rls)	((rls)->error)
#define	rls32fGetForgetFactor(rls)		((rls)->forgetFactor)

/* API to set the properties of an asptRls32f filter */
DT32S	rls32fReset(asptRls32f *rls);
DT32S	rls32fResetDelayLine(asptRls32f *rls);
DT32S	rls32fResize(asptRls32f *rls, DT32S newLength);
DT32S	rls32fSetCoef(asptRls32f *rls, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	rls32fSetDelayLine(asptRls32f *rls, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	rls32fSetForgetFactor(rls, newFactor) ((rls)->forgetFactor = newFactor)


#ifdef __cplusplus
}
#endif
#endif /* ASPTRLS_H_ */

