/*  ********************************************************************
    ASPTRLSLATTICE.H 

    Declares the Recursive Least Squares Lattice joint process estimator
    using the a posteriori estimation errors.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTRLSLATTICE_H_
#define ASPTRLSLATTICE_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon32f.h"

typedef struct _asptRlslattice32f {
	DT32F	*ff;			/* autocorrelation vector of forward prediction error  */
	DT32F	*bb;			/* autocorrelation vector of backward prediction error  */
	DT32F	*fb;			/* cross-correlation of f and b  */
	DT32F	*be;			/* cross-correlation of b and e  */
	DT32F	*cf;			/* conversion factor */
	DT32F	*b;				/* vector of backward prediction error  */
	DT32F	*kf;			/* forward lattice coefficients  */
	DT32F	*kb;			/* backward lattice coefficients */
	DT32F	*c;				/* linear combiner coefficients  */
	DT32S	filterLength;	/* linear combiner length in samples  */
	DT32F	forgetFactor;	/* forgetting factor  */
	DT32F	error;			/* last error sample           */
} asptRlslattice32f;

/* API to create and delete an asptRlslattice32f filter */
DT32S	rlslattice32fInit(asptRlslattice32f *rlsl, DT32S L, DT32F mu);
DT32S	rlslattice32fInitStatic(asptRlslattice32f *rlsl, DT32S L, DT32F mu, DT32F *pCof, DT32F *pDat);
void	rlslattice32fFree(asptRlslattice32f *rlsl);

/* APT to filter and update an asptRlslattice32f filter */
DT32F	rlslattice32fFilterUpdate(asptRlslattice32f *rlsl, DT32F x, DT32F d);
DT32F	rlslattice32fFilterOnly(asptRlslattice32f *rlsl, DT32F x);

/* API to retreve the properties of an asptRlslattice32f filter */
DT32S	rlslattice32fGetCoef(asptRlslattice32f *rlsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslattice32fGetForwardParcors(asptRlslattice32f *rlsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslattice32fGetBackwardParcors(asptRlslattice32f *rlsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslattice32fGetDelayLine(asptRlslattice32f *rlsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
#define	rlslattice32fGetCoefPtr(rlsl)			((rlsl)->c)
#define	rlslattice32fGetForwardParcorPtr(rlsl)	((rlsl)->kf)
#define	rlslattice32fGetBackwardParcorPtr(rlsl)	((rlsl)->kb)
#define	rlslattice32fGetDelayLinePtr(rlsl)		((rlsl)->b)
#define	rlslattice32fGetFilterLength(rlsl)		((rlsl)->filterLength)
#define	rlslattice32fGetForgetFactor(rlsl)		((rlsl)->forgetFactor)
#define	rlslattice32fGetLastErrorSample(rlsl)	((rlsl)->error)

/* API to set the properties of an asptRlslattice32f filter */
DT32S	rlslattice32fReset(asptRlslattice32f *rlsl);
DT32S	rlslattice32fResize(asptRlslattice32f *rlsl, DT32S newLength);
DT32S	rlslattice32fSetDelayLine(asptRlslattice32f *rlsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
#define	rlslattice32fSetForgetFactor(rlsl, newFactor) ((rlsl)->forgetFactor = newFactor)

#ifdef __cplusplus
}
#endif
#endif /* ASPTRLSLATTICE_H_ */

