/*  ********************************************************************
    ASPTRLSLBPEF.H 

    Declares Recursive Least Squares Lattice Backward Prediction Error Filter
    using the a posteriori estimation errors.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTRLSLBPEF_H_
#define ASPTRLSLBPEF_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon32f.h"

typedef struct _asptRlslbpef32f {
	DT32F	*ff;			/* autocorrelation vector of forward prediction error  */
	DT32F	*bb;			/* autocorrelation vector of backward prediction error  */
	DT32F	*fb;			/* cross-correlation of f and b  */
/*	DT32F	*be;			// cross-correlation of b and e  */
	DT32F	*cf;			/* conversion factor */
	DT32F	*b;				/* vector of backward prediction error  */
	DT32F	*kf;			/* forward lattice coefficients  */
	DT32F	*kb;			/* backward lattice coefficients */
	DT32F	*c;				/* linear combiner coefficients  */
	DT32F	*inBuf;			/* input signal buffer */
	DT32S		index;			/* index of last inserted input sample */
	DT32S	filterLength;	/* linear combiner length in samples  */
	DT32F	forgetFactor;	/* forgetting factor  */
	DT32F	error;			/* last error sample   */
	DT32S		firFlag;		/* if non-zero will calculate the equivalent transversal */
							/* coefficients and store them in c each iteration. */
} asptRlslbpef32f;

/* API to create and delete an asptRlslbpef32f filter */
DT32S	rlslbpef32fInit(asptRlslbpef32f *rlsl, DT32S L, DT32F mu);
DT32S	rlslbpef32fInitStatic(asptRlslbpef32f *rlsl, DT32S L, DT32F mu, DT32F *pCof, DT32F *pDat);
void	rlslbpef32fFree(asptRlslbpef32f *rlsl);

/* APT to filter and update an asptRlslbpef32f filter */
DT32F	rlslbpef32fFilterUpdate(asptRlslbpef32f *rlsl, DT32F x);
DT32F	rlslbpef32fFilterOnly(asptRlslbpef32f *rlsl, DT32F x);

/* API to retreve the properties of an asptRlslbpef32f filter */
DT32S	rlslbpef32fGetCoef(asptRlslbpef32f *rlsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslbpef32fGetForwardParcors(asptRlslbpef32f *rlsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslbpef32fGetBackwardParcors(asptRlslbpef32f *rlsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslbpef32fGetDelayLine(asptRlslbpef32f *rlsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
#define	rlslbpef32fGetCoefPtr(rlsl)				((rlsl)->c)
#define	rlslbpef32fGetForwardParcorPtr(rlsl)	((rlsl)->kf)
#define	rlslbpef32fGetBackwardParcorPtr(rlsl)	((rlsl)->kb)
#define	rlslbpef32fGetDelayLinePtr(rlsl)		((rlsl)->b)
#define	rlslbpef32fGetFilterLength(rlsl)		((rlsl)->filterLength)
#define	rlslbpef32fGetForgetFactor(rlsl)		((rlsl)->forgetFactor)
#define	rlslbpef32fGetLastErrorSample(rlsl)		((rlsl)->error)
#define	rlslbpef32fGetFirFlag(rlsl)				((rlsl)->firFlag)

/* API to set the properties of an asptRlslbpef32f filter */
DT32S	rlslbpef32fReset(asptRlslbpef32f *rlsl);
DT32S	rlslbpef32fResize(asptRlslbpef32f *rlsl, DT32S newLength);
DT32S	rlslbpef32fSetDelayLine(asptRlslbpef32f *rlsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
#define	rlslbpef32fSetForgetFactor(rlsl, newFactor) ((rlsl)->forgetFactor = newFactor)
#define	rlslbpef32fSetFirFlag(rlsl, newFlag)		((rlsl)->firFlag = newFlag)

#ifdef __cplusplus
}
#endif
#endif /* ASPTRLSLBPEF_H_ */

