/*  ********************************************************************
    ASPTRLSLFPEF.H 

    Declares Recursive Least Squares Lattice Forward Prediction Error 
	Filter using the a posteriori estimation errors.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTRLSLFPEF_H_
#define ASPTRLSLFPEF_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon32f.h"

typedef struct _asptRlslfpef32f {
	DT32F	*ff;			/* autocorrelation vector of forward prediction error  */
	DT32F	*bb;			/* autocorrelation vector of backward prediction error  */
	DT32F	*fb;			/* cross-correlation of f and b  */
/*	DT32F	*be;			// cross-correlation of b and e  */
	DT32F	*cf;			/* conversion factor */
	DT32F	*b;				/* vector of backward prediction error  */
	DT32F	*kf;			/* forward lattice coefficients  */
	DT32F	*kb;			/* backward lattice coefficients */
	DT32F	*c;				/* linear combiner coefficients  */
	DT32S	filterLength;	/* linear combiner length in samples  */
	DT32F	forgetFactor;	/* forgetting factor  */
	DT32F	error;			/* last error sample   */
	DT32S	firFlag;		/* if non-zero will calculate the equivalent transversal */
							/* coefficients and store them in c each iteration. */
} asptRlslfpef32f;

/* API to create and delete an asptRlslfpef32f filter */
DT32S	rlslfpef32fInit(asptRlslfpef32f *rlsl, DT32S L, DT32F mu);
DT32S	rlslfpef32fInitStatic(asptRlslfpef32f *rlsl, DT32S L, DT32F mu, DT32F *pCof, DT32F *pDat);
void	rlslfpef32fFree(asptRlslfpef32f *rlsl);

/* APT to filter and update an asptRlslfpef32f filter */
DT32F	rlslfpef32fFilterUpdate(asptRlslfpef32f *rlsl, DT32F x);
DT32F	rlslfpef32fFilterOnly(asptRlslfpef32f *rlsl, DT32F x);

/* API to retreve the properties of an asptRlslfpef32f filter */
DT32S	rlslfpef32fGetCoef(asptRlslfpef32f *rlsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslfpef32fGetForwardParcors(asptRlslfpef32f *rlsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslfpef32fGetBackwardParcors(asptRlslfpef32f *rlsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslfpef32fGetDelayLine(asptRlslfpef32f *rlsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
#define	rlslfpef32fGetCoefPtr(rlsl)				((rlsl)->c)
#define	rlslfpef32fGetForwardParcorPtr(rlsl)	((rlsl)->kf)
#define	rlslfpef32fGetBackwardParcorPtr(rlsl)	((rlsl)->kb)
#define	rlslfpef32fGetDelayLinePtr(rlsl)		((rlsl)->b)
#define	rlslfpef32fGetFilterLength(rlsl)		((rlsl)->filterLength)
#define	rlslfpef32fGetForgetFactor(rlsl)		((rlsl)->forgetFactor)
#define	rlslfpef32fGetLastErrorSample(rlsl)		((rlsl)->error)
#define	rlslfpef32fGetFirFlag(rlsl)				((rlsl)->firFlag)

/* API to set the properties of an asptRlslfpef32f filter */
DT32S	rlslfpef32fReset(asptRlslfpef32f *rlsl);
DT32S	rlslfpef32fResize(asptRlslfpef32f *rlsl, DT32S newLength);
DT32S	rlslfpef32fSetDelayLine(asptRlslfpef32f *rlsl, DT32F *buf, DT32S ind, DT32S N, DT32S flip);
#define	rlslfpef32fSetForgetFactor(rlsl, newFactor) ((rlsl)->forgetFactor = newFactor)
#define	rlslfpef32fSetFirFlag(rlsl, newFlag)		((rlsl)->firFlag = newFlag)

#ifdef __cplusplus
}
#endif
#endif /* ASPTRLSLFPEF_H_ */

