/*  ********************************************************************
    ASPTTDLMS.C 
    Declares the Transform-Domain LMS Adaptive Filter (TDLMS).
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef TDLMS_H_
#define TDLMS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon32f.h"
#include "asptfft32f.h"

typedef struct _asptTdlms32f {
	DT32S	filterLength;			/* filter length in samples */
	DT32S	fftLength;				/* fft length               */	
	DT32S	BO2P1;					/* length of complex vector */
	DT32F	stepSize;				/* TDLMS step size          */
	DT32F	error;					/* last error sample        */
	DT32F	scale;					/* scaling factor           */
	DT32S	idx;
	DT32F	*xTime;					/* input vec. in T-domain   */
	DT32F	*xFreq;					/* input vec. in F-domain   */
	DT32F	*wFreq;					/* coef. vector F-domain	*/
	DT32F	*pFreq;					/* input signal power       */
	DT32F	*pNew;					/* new input power          */
	DT32F	*w;						/* FFT twiddle factor table */
	DT32S	*ip;					/* work buffer for FFT      */
} asptTdlms32f;

/* API to create and delete asptTdlms32f filters */
DT32S	tdlms32fInit(asptTdlms32f *tdlms,DT32S L,DT32F mu,DT32S *ip,DT32F *w);
DT32S	tdlms32fInitStatic(asptTdlms32f *tdlms,DT32S L,DT32F mu,DT32F *pCof,DT32F *pDat,DT32S *ip,DT32F *w);
void	tdlms32fFree(asptTdlms32f *tdlms);


/* API to filter and update asptTdlms32f filters */
DT32F tdlms32fFilterUpdate(asptTdlms32f *tdlms, DT32F inp, DT32F des);
DT32F tdlms32fFilterOnly(asptTdlms32f *tdlms, DT32F inp);
DT32F tdlms32fLCFilterUpdate(asptTdlms32f *tdlms, DT32F des);
DT32F tdlms32fLCFilterOnly(asptTdlms32f *tdlms);

/* API to retrieve the properties of asptTdlms32f filters */
DT32S	tdlms32fGetCoef(asptTdlms32f *tdlms, DT32F *dstBuf, DT32S ind, DT32S N, DT32S flip);
DT32S	tdlms32fGetDelayLine(asptTdlms32f *tdlms, DT32F *dstBuf, DT32S ind, DT32S N, DT32S flip);
#define	tdlms32fGetCoefPtr(tdlms) 			((tdlms)->wFreq)
#define	tdlms32fGetFilterLength(tdlms) 		((tdlms)->filterLength)
#define	tdlms32fGetFftLength(tdlms) 		((tdlms)->fftLength)
#define	tdlms32fGetDelayLineLength(tdlms) 	((tdlms)->fftLength)
#define	tdlms32fGetStepSize(tdlms) 			((tdlms)->stepSize)
#define	tdlms32fGetLastErrorSample(tdlms) 	((tdlms)->error)

/* API to set the properties of asptTdlms32f filters */
DT32S	tdlms32fReset(asptTdlms32f *tdlms);
DT32S	tdlms32fResetDelayLine(asptTdlms32f *tdlms);
DT32S	tdlms32fResize(asptTdlms32f *tdlms, DT32S newLen);
DT32S	tdlms32fSetCoef(asptTdlms32f *tdlms, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	tdlms32fSetDelayLine(asptTdlms32f *tdlms, DT32F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	tdlms32fSetStepSize(tdlms, newSs) 	((tdlms)->stepSize = newSs)


#ifdef __cplusplus
}
#endif

#endif /* TDLMS_H_ */
