/*  ********************************************************************
    ASPTADJLMS.H 

    Declares the Adjoint LMS algorithm.
    This version is intended for simulation purposes and matches 
    the Matlab asptadjlms and init_adjlms functions.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTADJLMS_H_
#define ASPTADJLMS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon64f.h"

typedef struct _asptAdjlms64f {
	DT32S	filterLength;			/* filter length in samples   */
	DT32S	secPathLength;			/* secondary path length      */
	DT32S	estSecPathLength;		/* estimated sec. path length */
	DT32S	refLength;				/* length of reference signal */
	DT64F	stepSize;				/* LMS step size              */
	DT64F	sensor;					/* last sensor sample         */
	DT64F	power;                  /* last estimated input power */
	DT64F	*coef;					/* filter coef. vector        */
	DT64F	*secPath;				/* secondary path             */
	DT64F	*secPathEst;			/* estimated secondary path   */
	DT64F	*yBuf;					/* buffer for output signal   */
	DT64F	*feBuf;					/* filtered-error signal      */
	DT64F	*delayLine;				/* pointer to the filter data */
	DT32S	xIndex;
	DT32S	yIndex;
	DT32S	eIndex;
} asptAdjlms64f;

/* API to create and delete an asptAdjlms64f filters */
DT32S	adjlms64fInit (asptAdjlms64f *adjlms,DT32S L, DT64F mu, DT64F *sPath, DT32S sL, DT64F *sPathEst, DT32S seL);
DT32S	adjlms64fInitStatic(asptAdjlms64f *adjlms,DT32S L, DT64F mu, DT64F *sPath, DT32S sL, DT64F *sPathEst, DT32S seL, DT64F *pCof, DT64F *pDat);
void	adjlms64fFree(asptAdjlms64f *adjlms);

/* API to filter through and update an asptAdjlms64f filters */
DT64F	adjlms64fFilterUpdate(asptAdjlms64f *adjlms, DT64F ref, DT64F prim);
DT64F	adjlms64fFilterOnly  (asptAdjlms64f *adjlms, DT64F ref);

/* API to retrieve the properties of an asptAdjlms64f filter */
DT32S	adjlms64fGetCoef(asptAdjlms64f *adjlms, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	adjlms64fGetDelayLine(asptAdjlms64f *adjlms, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
#define	adjlms64fGetCoefPtr(adjlms)				((adjlms)->coef)
#define	adjlms64fGetSecPathPtr(adjlms)			((adjlms)->secPath)
#define	adjlms64fGetEstSecPathPtr(adjlms)		((adjlms)->secPathEst)
#define	adjlms64fGetFilterLength(adjlms)		((adjlms)->filterLength)
#define	adjlms64fGetSecPathLength(adjlms)		((adjlms)->secPathLength)
#define	adjlms64fGetEstSecPathLength(adjlms)	((adjlms)->estSecPathLength)
#define	adjlms64fGetLastSensorSample(adjlms)	((adjlms)->sensor)
#define	adjlms64fGetStepSize(adjlms)			((adjlms)->stepSize)


/* API to set the properties of an asptAdjlms64f filter */
DT32S	adjlms64fReset(asptAdjlms64f *adjlms);
DT32S	adjlms64fResetDelayLine(asptAdjlms64f *adjlms);
DT32S	adjlms64fResize(asptAdjlms64f *adjlms, DT32S newLength);
DT32S	adjlms64fSetCoef(asptAdjlms64f *adjlms, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	adjlms64fSetDelayLine(asptAdjlms64f *adjlms, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	adjlms64fSetStepSize(adjlms, newStep)			((adjlms)->stepSize = newStep)
#define	adjlms64fSetSecPathPtr(adjlms, newPtr)			((adjlms)->secPath = newPtr)
#define	adjlms64fSetEstSecPathPtr(adjlms, newPtr)		((adjlms)->secPathEst = newPtr)
#define	adjlms64fSetSecPathLength(adjlms, newLength)	((adjlms)->secPathLength = newLength)
#define	adjlms64fSetEstSecPathLength(adjlms, newLength)	((adjlms)->estSecPathLength = newLength)

#ifdef __cplusplus
}
#endif

#endif /* ASPTADJLMS_H_ */
