/*  ********************************************************************
    ASPTBFDAF.C 
    Declares the Block Frequency Domain Adaptive Filter (BFDAF).
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef BFDAF_H_
#define BFDAF_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon64f.h"
#include "asptfft64f.h"

typedef struct _asptBfdaf64f {
	DT32S	filterLength;			/* filter length in samples */
	DT32S	blockLength;			/* new samples per block    */
	DT32S	fftLength;				/* fft length               */	
	DT32S	BO2P1;					/* length of complex vector */
	DT32S	constrain;				/* constrained BFDAF        */
	DT32S	C1;						/* fftLength - blockLength  */
	DT32S	C2;						/* fftLength - filterLength */
	DT64F	B2;						/* power smoothing pole     */

	DT64F	stepSize;				/* BFDAF step size          */
	DT64F	scale;					/* scaling factor           */
	DT64F	*xTime;					/* input vec. in T-domain   */
	DT64F	*xFreq;					/* input vec. in F-domain   */
	DT64F	*wFreq;					/* coef. vector F-domain	*/
	DT64F	*yFreq;					/* filter output            */
	DT64F	*pFreq;					/* input signal power       */
	DT64F	*pNew;					/* new input power          */
	DT64F	*w;						/* FFT twiddle factor table */
	DT32S	*ip;					/* work buffer for FFT      */
} asptBfdaf64f;

/* API to create and delete asptBfdaf64f filters */
DT32S	bfdaf64fInit(asptBfdaf64f *bfdaf,DT32S M,DT32S L,DT64F mu,DT32S cs,DT32S *ip,DT64F *w);
DT32S	bfdaf64fInitStatic(asptBfdaf64f *bfdaf,DT32S M,DT32S L,DT64F mu,DT32S cs,DT64F *pCof,DT64F *pDat,DT32S *ip,DT64F *w);
void	bfdaf64fFree(asptBfdaf64f *bfdaf);


/* API to filter and update asptBfdaf64f filters */
DT32S bfdaf64fFilterUpdate(asptBfdaf64f *bfdaf, DT64F *inp, DT64F *des, DT64F *out, DT64F *err);
DT32S bfdaf64fFilterOnly(asptBfdaf64f *bfdaf, DT64F *inp, DT64F *out);

/* APT to retrieve the properties of asptBfdaf64f filters */
DT32S	bfdaf64fGetCoef(asptBfdaf64f *bfdaf, DT64F *dstBuf, DT32S ind, DT32S N, DT32S flip);
#define	bfdaf64fGetCoefPtr(bfdaf) 			((bfdaf)->wFreq)
#define	bfdaf64fGetFilterLength(bfdaf) 		((bfdaf)->filterLength)
#define	bfdaf64fGetBlockLength(bfdaf) 		((bfdaf)->blockLength)
#define	bfdaf64fGetFftLength(bfdaf) 		((bfdaf)->fftLength)
#define	bfdaf64fGetStepSize(bfdaf) 			((bfdaf)->stepSize)
#define	bfdaf64fGetConstrain(bfdaf) 		((bfdaf)->constrain)

/* API to set the properties of asptBfdaf64f filters */
DT32S	bfdaf64fReset(asptBfdaf64f *bfdaf);
DT32S	bfdaf64fResetDelayLine(asptBfdaf64f *bfdaf);
DT32S	bfdaf64fResize(asptBfdaf64f *bfdaf, DT32S newLen, DT32S newBlock);
DT32S	bfdaf64fSetCoef(asptBfdaf64f *bfdaf, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	bfdaf64fSetStepSize(bfdaf, newSs) 	((bfdaf)->stepSize = newSs)
#define	bfdaf64fSetConstrain(bfdaf, newC) 	((bfdaf)->constrain = newC)


#ifdef __cplusplus
}
#endif

#endif /* BFDAF_H_ */
