/*  ********************************************************************
    ASPTBLMS.H 

    Declares the Block Least Mean Squares (BLMS) adaptive filter algorithm.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTBLMS_H_
#define ASPTBLMS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon64f.h"

typedef struct _asptBlms64f {
	DT32S	filterLength;			/* filter length in samples		*/
	DT32S	blockLength;			/* new samples per block		*/
	DT64F	stepSize;				/* LMS step size				*/
	DT64F	*coef;					/* filter coef. vector			*/
	DT64F	*delayLine;				/* pointer to the filter data	*/
	DT32S	xIndex;
} asptBlms64f;

/* API to create and delete an asptBlms64f filters */
DT32S	blms64fInit (asptBlms64f *blms,DT32S L, DT32S B, DT64F mu);
DT32S	blms64fInitStatic(asptBlms64f *blms,DT32S L, DT32S B, DT64F mu, DT64F *pCof, DT64F *pDat);
void	blms64fFree(asptBlms64f *blms);

/* API to filter through and update an asptBlms64f filters */
DT32S	blms64fFilterUpdate        (asptBlms64f *blms, DT64F *inp, DT64F *des, DT64F *out, DT64F *err);
DT32S	blms64fFilterUpdateSignErr (asptBlms64f *blms, DT64F *inp, DT64F *des, DT64F *out, DT64F *err);
DT32S	blms64fFilterUpdateSignReg (asptBlms64f *blms, DT64F *inp, DT64F *des, DT64F *out, DT64F *err);
DT32S	blms64fFilterUpdateSignSign(asptBlms64f *blms, DT64F *inp, DT64F *des, DT64F *out, DT64F *err);
DT32S	blms64fFilterOnly  (asptBlms64f *blms, DT64F *inp, DT64F *out);

/* API to retrieve the properties of an asptBlms64f filter */
DT32S	blms64fGetCoef(asptBlms64f *blms, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	blms64fGetDelayLine(asptBlms64f *blms, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
#define	blms64fGetCoefPtr(blms)			((blms)->coef)
#define	blms64fGetFilterLength(blms)	((blms)->filterLength)
#define	blms64fGetStepSize(blms)		((blms)->stepSize)
#define	blms64fGetBlockLength(blms)		((blms)->blockLength)

/* API to set the properties of an asptBlms64f filter */
DT32S	blms64fReset(asptBlms64f *blms);
DT32S	blms64fResetDelayLine(asptBlms64f *blms);
DT32S	blms64fResize(asptBlms64f *blms, DT32S newL, DT32S newB);
DT32S	blms64fSetCoef(asptBlms64f *blms, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	blms64fSetDelayLine(asptBlms64f *blms, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	blms64fSetStepSize(blms, newStep)	((blms)->stepSize = newStep)
#define	blms64fSetBlockLength(blms, newBlock) ((blms)->blockLength = newBlock)

#ifdef __cplusplus
}
#endif

#endif /* ASPTBLMS_H_ */
