/*  ********************************************************************
    ASPTBNLMS.H 

    Declares the Block Normalized Least Mean Squares (BNLMS) adaptive 
	filter algorithm.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTBNLMS_H_
#define ASPTBNLMS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon64f.h"

typedef struct _asptBnlms64f {
	DT32S	filterLength;			/* filter length in samples		*/
	DT32S	blockLength;			/* new samples per block		*/
	DT64F	stepSize;				/* LMS step size				*/
	DT64F	*coef;					/* filter coef. vector			*/
	DT64F	power;					/* input signal power			*/
	DT64F	*delayLine;				/* pointer to the filter data	*/
	DT32S	xIndex;
} asptBnlms64f;

/* API to create and delete an asptBnlms64f filters */
DT32S	bnlms64fInit (asptBnlms64f *bnlms,DT32S L, DT32S B, DT64F mu);
DT32S	bnlms64fInitStatic(asptBnlms64f *bnlms,DT32S L, DT32S B, DT64F mu, DT64F *pCof, DT64F *pDat);
void	bnlms64fFree(asptBnlms64f *bnlms);

/* API to filter through and update an asptBnlms64f filters */
DT32S	bnlms64fFilterUpdate        (asptBnlms64f *bnlms, DT64F *inp, DT64F *des, DT64F *out, DT64F *err);
DT32S	bnlms64fFilterOnly  (asptBnlms64f *bnlms, DT64F *inp, DT64F *out);

/* API to retrieve the properties of an asptBnlms64f filter */
DT32S	bnlms64fGetCoef(asptBnlms64f *bnlms, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	bnlms64fGetDelayLine(asptBnlms64f *bnlms, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
#define	bnlms64fGetCoefPtr(bnlms)			((bnlms)->coef)
#define	bnlms64fGetFilterLength(bnlms)	((bnlms)->filterLength)
#define	bnlms64fGetStepSize(bnlms)		((bnlms)->stepSize)
#define	bnlms64fGetBlockLength(bnlms)		((bnlms)->blockLength)

/* API to set the properties of an asptBnlms64f filter */
DT32S	bnlms64fReset(asptBnlms64f *bnlms);
DT32S	bnlms64fResetDelayLine(asptBnlms64f *bnlms);
DT32S	bnlms64fResize(asptBnlms64f *bnlms, DT32S newL, DT32S newB);
DT32S	bnlms64fSetCoef(asptBnlms64f *bnlms, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	bnlms64fSetDelayLine(asptBnlms64f *bnlms, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	bnlms64fSetStepSize(bnlms, newStep)	((bnlms)->stepSize = newStep)
#define	bnlms64fSetBlockLength(bnlms, newBlock) ((bnlms)->blockLength = newBlock)

#ifdef __cplusplus
}
#endif

#endif /* ASPTBNLMS_H_ */
