/*  ********************************************************************
    COMMON.H 

    declares functions common to more than one algorithm in the package.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef COMMON_H_
#define COMMON_H_

#include <math.h>
#include <stdlib.h>
#include "asptdefs.h"

#ifdef __cplusplus
extern "C" {
#endif


/* Length calculation functions */
DT32S	nextPow2(const DT64S N);
DT32S	rsum(DT32S x, DT32S y);

/* DT64F vector operations */
DT64F	fir64f(DT64F s, DT64F x[], const DT64F h[], DT32S xLen, DT32S hLen, DT32S *pos);
DT64F	firAdj64f(DT64F s, DT64F x[], const DT64F h[], DT32S xLen, DT32S hLen, DT32S *pos);
DT64F	dotProdCircBuf64f(DT64F x[], const DT64F h[], DT32S xLen, DT32S hLen, DT32S pos);
DT64F	dotProdLinBuf64f(const DT64F x[], const DT64F h[], DT32S N);
DT32S	sum64f(DT64F *src,DT64F *sum, DT32S N);
DT32S	sumAbs64f(DT64F *src,DT64F *sum, DT32S N);
DT32S	copy64f(const DT64F *src, DT64F *dst, DT32S N);
DT32S	set64f(DT64F g, DT64F *srcdst, DT32S N);
DT32S	reset64f(DT64F *srcdst, DT32S N);
DT32S	scaleAdd64f(DT64F scale, const DT64F *src1, const DT64F *src2, DT64F *dst, DT32S N);
DT32S	scaleAddIp64f(DT64F scale, const DT64F *src, DT64F *srcdst, DT32S N);
DT32S	smooth64f(DT64F a1, DT64F *src1, DT64F a2, DT64F *srcdst, DT32S N);
DT32S	add64f(const DT64F *src1, const DT64F *src2, DT64F *dst, DT32S N);
DT32S	addIp64f(const DT64F *src, DT64F *srcdst, DT32S N);
DT32S	sub64f(const DT64F *src1, const DT64F *src2, DT64F *dst, DT32S N);
DT32S	scaleIp64f(DT64F scale, DT64F *srcdst, DT32S N);
DT32S	divIp64f(const DT64F *src, DT64F d, DT64F *srcdst, DT32S N);
DT32S	div64f(const DT64F *src1, const DT64F *src2, DT64F d, DT64F *dst, DT32S N);
DT32S	insertSampleLinBuf64f(DT64F x, DT64F *buf, DT32S N);

/* Complex DT64FC vector operations */
DT32S	powerSpectr64fc(const DT64FC *src, DT64F *dst, DT32S N);
DT32S	powerSpectrAdd64fc(const DT64FC *src, DT64F *srcdst, DT32S N);
DT32S	conjIp64fc(DT64FC *srcdst, DT32S N);
DT32S	conj64fc(DT64FC *src, DT64FC *dst, DT32S N);
DT32S	mpy64fc(const DT64FC *src1, const DT64FC *src2, DT64FC *dst, DT32S N);
DT32S	mpyIp64fc(const DT64FC *src, DT64FC *srcdst, DT32S N);
DT32S	mpyIp64f64fc(const DT64F *src, DT64FC *srcdst, DT32S N);
DT32S	mpyAdd64fc(const DT64FC *src1, const DT64FC *src2, DT64FC *srcdst, DT32S N);
DT32S	mpySub64fc(const DT64FC *src1, const DT64FC *src2, DT64FC *srcdst, DT32S N);
DT32S	divIp64f64fc(const DT64F *src, DT64F d, DT64FC *srcdst, DT32S N);
DT32S	subIp64fc(const DT64FC *src, DT64FC *srcdst, DT32S N);
DT32S	scaleIp64fc(DT64F scale, DT64FC *srcdst, DT32S N);

/* Half complex DT64FC vector operations */
DT32S	powerSpectrR64fc(const DT64FC *src, DT64F *dst, DT32S N);
DT32S	powerSpectrAddR64fc(const DT64FC *src, DT64F *srcdst, DT32S N);
DT32S	conjIpR64fc(DT64FC *srcdst, DT32S N);
DT32S	conjR64fc(DT64FC *src, DT64FC *dst, DT32S N);
DT32S	mpyR64fc(const DT64FC *src1, const DT64FC *src2, DT64FC *dst, DT32S N);
DT32S	mpyIpR64fc(const DT64FC *src, DT64FC *srcdst, DT32S N);
DT32S	mpyIp64fR64fc(const DT64F *src, DT64FC *srcdst, DT32S N);
DT32S	mpyAddR64fc(const DT64FC *src1, const DT64FC *src2, DT64FC *srcdst, DT32S N);
DT32S	mpySubR64fc(const DT64FC *src1, const DT64FC *src2, DT64FC *srcdst, DT32S N);
DT32S	divIp64fR64fc(const DT64F *src, DT64F d, DT64FC *srcdst, DT32S N);
DT32S	subIpR64fc(const DT64FC *src, DT64FC *srcdst, DT32S N);
DT32S	scaleIpR64fc(DT64F scale, DT64FC *srcdst, DT32S N);


#ifdef __cplusplus
}
#endif

#endif /* COMMON_H_ */


