/*  ********************************************************************
    FIR.H 

    Declares a Finite Impulse Response filter.

    This version uses a circular buffer of length equals to the filter 
	length.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */

#ifndef FIR_H_
#define FIR_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon64f.h"

typedef struct _asptFir64f {
	DT64F	*coef;			/* coefficient vector	      */
	DT32S	filterLength;	/* coef vector length	      */
	DT64F	*delayLine;		/* circular buffer		      */
	DT32S	idx;			/* index to the oldest sample */
} asptFir64f;

/* API to create and delete asptFir64f filters */
DT32S	fir64fInit (asptFir64f *fir, DT32S L, DT64F *pCof);
DT32S	fir64fInitStatic(asptFir64f *fir, DT32S L, DT64F *pCof, DT64F *pDat);
void	fir64fFree(asptFir64f *fir);

/* API to filter through asptFir64f filters */
DT64F	fir64fFilter(asptFir64f *fir, DT64F inSample);
DT64F	fir64fLCFilter(asptFir64f *fir);
DT32S	fir64fBlockFilter(asptFir64f *fir, DT64F *in, DT64F *out,DT32S BlockLength);

/* API to retrieve the properties of an asptFir64f filter */
DT32S	fir64fGetCoef(asptFir64f *fir, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	fir64fGetDelayLine(asptFir64f *fir, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
#define	fir64fGetCoefPtr(fir)			((fir)->coef)
#define	fir64fGetFilterLength(fir)		((fir)->filterLength)
#define	fir64fGetDelayLineLength(fir)	((fir)->filterLength)

/* API to set the properties of an asptFir64f filter */
DT32S	fir64fResetDelayLine(asptFir64f *fir);
DT32S	fir64fResize(asptFir64f *fir, DT32S newLength);
DT32S	fir64fSetCoef(asptFir64f *fir, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	fir64fSetDelayLine(asptFir64f *fir, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);

#ifdef __cplusplus
}
#endif

#endif /* FIR_H_ */
