/*  ********************************************************************
    ASPTFXLMS.H 

    Declares the Filtered-X LMS algorithm.
    This version is intended for simulation purposes and matches 
    the Matlab asptfxlms and init_fxlms functions.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTFXLMS_H_
#define ASPTFXLMS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon64f.h"

typedef struct _asptFxlms64f {
	DT32S	filterLength;			/* filter length in samples		*/
	DT32S	secPathLength;			/* secondary path length		*/
	DT32S	estSecPathLength;		/* estimated sec. path length	*/
	DT32S	refLength;				/* input signal buffer length   */
	DT64F	stepSize;				/* LMS step size				*/
	DT64F	sensor;					/* last sensor sample			*/
	DT64F	power;                  /* last estimated input power	*/
	DT64F	*coef;					/* filter coef. vector			*/
	DT64F	*secPath;				/* secondary path				*/
	DT64F	*secPathEst;			/* estimated secondary path		*/
	DT64F	*yBuf;					/* buffer for output signal		*/
	DT64F	*fxBuf;					/* filtered-x signalbuffer		*/
	DT64F	*delayLine;				/* pointer to the filter data	*/
	DT32S	xIndex;
	DT32S	yIndex;
	DT32S	fIndex;
} asptFxlms64f;

/* API to create and delete an asptFxlms64f filters */
DT32S	fxlms64fInit (asptFxlms64f *fxlms,DT32S L, DT64F mu, DT64F *sPath, DT32S sL, DT64F *sPathEst, DT32S seL);
DT32S	fxlms64fInitStatic(asptFxlms64f *fxlms,DT32S L, DT64F mu, DT64F *sPath, DT32S sL, DT64F *sPathEst, DT32S seL, DT64F *pCof, DT64F *pDat);
void	fxlms64fFree(asptFxlms64f *fxlms);

/* API to filter through and update an asptFxlms64f filters */
DT64F	fxlms64fFilterUpdate(asptFxlms64f *fxlms, DT64F ref, DT64F prim);
DT64F	fxlms64fFilterOnly  (asptFxlms64f *fxlms, DT64F ref);

/* API to retrieve the properties of an asptFxlms64f filter */
DT32S	fxlms64fGetCoef(asptFxlms64f *fxlms, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	fxlms64fGetDelayLine(asptFxlms64f *fxlms, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
#define	fxlms64fGetCoefPtr(fxlms)			((fxlms)->coef)
#define	fxlms64fGetSecPathPtr(fxlms)		((fxlms)->secPath)
#define	fxlms64fGetEstSecPathPtr(fxlms)		((fxlms)->secPathEst)
#define	fxlms64fGetFilterLength(fxlms)		((fxlms)->filterLength)
#define	fxlms64fGetSecPathLength(fxlms)		((fxlms)->secPathLength)
#define	fxlms64fGetEstSecPathLength(fxlms)	((fxlms)->estSecPathLength)
#define	fxlms64fGetLastSensorSample(fxlms)	((fxlms)->sensor)
#define	fxlms64fGetStepSize(fxlms)			((fxlms)->stepSize)


/* API to set the properties of an asptFxlms64f filter */
DT32S	fxlms64fReset(asptFxlms64f *fxlms);
DT32S	fxlms64fResetDelayLine(asptFxlms64f *fxlms);
DT32S	fxlms64fResize(asptFxlms64f *fxlms, DT32S newLength);
DT32S	fxlms64fSetCoef(asptFxlms64f *fxlms, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	fxlms64fSetDelayLine(asptFxlms64f *fxlms, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	fxlms64fSetStepSize(fxlms, newStep)				((fxlms)->stepSize = newStep)
#define	fxlms64fSetSecPathPtr(fxlms, newPtr)			((fxlms)->secPath = newPtr)
#define	fxlms64fSetEstSecPathPtr(fxlms, newPtr)			((fxlms)->secPathEst = newPtr)
#define	fxlms64fSetSecPathLength(fxlms, newLength)		((fxlms)->secPathLength = newLength)
#define	fxlms64fSetEstSecPathLength(fxlms, newLength)	((fxlms)->estSecPathLength = newLength)

#ifdef __cplusplus
}
#endif

#endif /* ASPTFXLMS_H_ */
