/*  ********************************************************************
    ASPTLBPEF.H 
    Declares the Least Mean Square adaptive Lattice Backward Prediction 
	Error Filter (LBPEF).
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */

#ifndef ASPTLBPEF_H_
#define ASPTLBPEF_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon64f.h"

typedef struct _asptLbpef64f {
	DT32S	filterLength;	/* filter length in samples  */
	DT64F	pcStepSize;		/* predictor coef. step size  */
	DT64F	error;			/* last error sample */
	DT32S	firFlag;		/* flag for calculating FIR coef. */
	DT32S	index;			/* index of last inserted input sample */
	DT64F	*parcor;		/* PARCOR coef. vector */
	DT64F	*coef;			/* linear combiner coef. vector */
	DT64F	*bpe;			/* backward prediction error vector */
	DT64F	*bpePower;		/* power estimate of bpe */
	DT64F	*scratch;		/* working buffer     */
	DT64F	*inBuf;			/* input signal buffer */
           
} asptLbpef64f;

/* API to create and delete an asptLbpef64f filter */
DT32S	lbpef64fInit(asptLbpef64f *lbpef, DT32S L, DT64F mu_p);
DT32S	lbpef64fInitStatic(asptLbpef64f *lbpef,DT32S L, DT64F mu_p, DT64F *pCof, DT64F *pDat);
void	lbpef64fFree(asptLbpef64f *lbpef);

/* APT to filter and update an asptLbpef64f filter */
DT64F	lbpef64fFilterUpdate(asptLbpef64f *lbpef, DT64F x);
DT64F	lbpef64fFilterOnly(asptLbpef64f *lbpef, DT64F x);

/* API to retreve the properties of an asptLbpef64f filter */
DT32S	lbpef64fGetCoef(asptLbpef64f *lbpef, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	lbpef64fGetParcor(asptLbpef64f *lbpef, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	lbpef64fGetDelayLine(asptLbpef64f *lbpef, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
#define	lbpef64fGetCoefPtr(lbpef)			((lbpef)->coef)
#define	lbpef64fGetParcorPtr(lbpef)			((lbpef)->parcor)
#define	lbpef64fGetDelayLinePtr(lbpef)		((lbpef)->bpe)
#define	lbpef64fGetFilterLength(lbpef)		((lbpef)->filterLength)
#define	lbpef64fGetPcStepSize(lbpef)		((lbpef)->pcStepSize)
#define	lbpef64fGetLastErrorSample(lbpef)	((lbpef)->error)
#define	lbpef64fGetFirFlag(lbpef)			((lbpef)->firFlag)

/* API to set the properties of an asptLbpef64f filter */
DT32S	lbpef64fReset(asptLbpef64f *lbpef);
DT32S	lbpef64fResize(asptLbpef64f *lbpef, DT32S newLength);
DT32S	lbpef64fSetCoef(asptLbpef64f *lbpef, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	lbpef64fSetParcor(asptLbpef64f *lbpef, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	lbpef64fSetDelayLine(asptLbpef64f *lbpef, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	lbpef64fSetPcStepSize(lbpef, newStep) ((lbpef)->pcStepSize = newStep)
#define	lbpef64fSetFirFlag(lbpef, newFlag) ((lbpef)->firFlag = newFlag)

#ifdef __cplusplus
}
#endif
#endif /* ASPTLBPEF_H_ */

