/*  ********************************************************************
    ASPTLFPEF.H 
    Declares the Least Mean Square adaptive Lattice Forward Prediction 
	Error Filter (LFPEF).
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */

#ifndef ASPTLFPEF_H_
#define ASPTLFPEF_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon64f.h"

typedef struct _asptLfpef64f {
	DT32S	filterLength;	/* filter length in samples  */
	DT64F	pcStepSize;		/* predictor coef. step size  */
	DT64F	error;			/* last error sample */
	DT32S	firFlag;		/* flag for calculating FIR coef. */
	DT32S	index;			/* index of last inserted input sample */
	DT64F	*parcor;		/* PARCOR coef. vector */
	DT64F	*coef;			/* linear combiner coef. vector */
	DT64F	*bpe;			/* backward prediction error vector */
	DT64F	*bpePower;		/* power estimate of bpe */
	DT64F	*scratch;		/* working buffer                */
} asptLfpef64f;

/* API to create and delete an asptLfpef64f filter */
DT32S	lfpef64fInit(asptLfpef64f *lfpef, DT32S L, DT64F mu_p);
DT32S	lfpef64fInitStatic(asptLfpef64f *lfpef,DT32S L, DT64F mu_p, DT64F *pCof, DT64F *pDat);
void	lfpef64fFree(asptLfpef64f *lfpef);

/* APT to filter and update an asptLfpef64f filter */
DT64F	lfpef64fFilterUpdate(asptLfpef64f *lfpef, DT64F x);
DT64F	lfpef64fFilterOnly(asptLfpef64f *lfpef, DT64F x);

/* API to retreve the properties of an asptLfpef64f filter */
DT32S	lfpef64fGetCoef(asptLfpef64f *lfpef, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	lfpef64fGetParcor(asptLfpef64f *lfpef, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	lfpef64fGetDelayLine(asptLfpef64f *lfpef, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
#define	lfpef64fGetCoefPtr(lfpef)			((lfpef)->coef)
#define	lfpef64fGetParcorPtr(lfpef)			((lfpef)->parcor)
#define	lfpef64fGetDelayLinePtr(lfpef)		((lfpef)->bpe)
#define	lfpef64fGetFilterLength(lfpef)		((lfpef)->filterLength)
#define	lfpef64fGetPcStepSize(lfpef)		((lfpef)->pcStepSize)
#define	lfpef64fGetLastErrorSample(lfpef)	((lfpef)->error)
#define	lfpef64fGetFirFlag(lfpef)			((lfpef)->firFlag)

/* API to set the properties of an asptLfpef64f filter */
DT32S	lfpef64fReset(asptLfpef64f *lfpef);
DT32S	lfpef64fResize(asptLfpef64f *lfpef, DT32S newLength);
DT32S	lfpef64fSetCoef(asptLfpef64f *lfpef, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	lfpef64fSetParcor(asptLfpef64f *lfpef, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	lfpef64fSetDelayLine(asptLfpef64f *lfpef, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	lfpef64fSetPcStepSize(lfpef, newStep) ((lfpef)->pcStepSize = newStep)
#define	lfpef64fSetFirFlag(lfpef, newFlag) ((lfpef)->firFlag = newFlag)

#ifdef __cplusplus
}
#endif
#endif /* ASPTLFPEF_H_ */

