/*  ********************************************************************
    ASPTLMS.H 

    Declares the Least Mean Squares (LMS) adaptive filter algorithm.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTLMS_H_
#define ASPTLMS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon64f.h"

typedef struct _asptLms64f {
	DT32S	filterLength;			/* filter length in samples		*/
	DT64F	stepSize;				/* LMS step size				*/
	DT64F	error;					/* last error sample			*/
	DT64F	*coef;					/* filter coef. vector			*/
	DT64F	*delayLine;				/* pointer to the filter data	*/
	DT32S	xIndex;
} asptLms64f;

/* API to create and delete an asptLms64f filters */
DT32S	lms64fInit (asptLms64f *lms,DT32S L, DT64F mu);
DT32S	lms64fInitStatic(asptLms64f *lms,DT32S L, DT64F mu, DT64F *pCof, DT64F *pDat);
void	lms64fFree(asptLms64f *lms);

/* API to filter through and update an asptLms64f filters */
DT64F	lms64fFilterUpdate        (asptLms64f *lms, DT64F inp, DT64F des);
DT64F	lms64fFilterUpdateSignErr (asptLms64f *lms, DT64F inp, DT64F des);
DT64F	lms64fFilterUpdateSignReg (asptLms64f *lms, DT64F inp, DT64F des);
DT64F	lms64fFilterUpdateSignSign(asptLms64f *lms, DT64F inp, DT64F des);
DT64F	lms64fFilterOnly  (asptLms64f *lms, DT64F inp);

DT64F	lms64fLCFilterUpdate        (asptLms64f *lms, DT64F des);
DT64F	lms64fLCFilterUpdateSignErr (asptLms64f *lms, DT64F des);
DT64F	lms64fLCFilterUpdateSignReg (asptLms64f *lms, DT64F des);
DT64F	lms64fLCFilterUpdateSignSign(asptLms64f *lms, DT64F des);
DT64F	lms64fLCFilterOnly  (asptLms64f *lms);

/* API to retrieve the properties of an asptLms64f filter */
DT32S	lms64fGetCoef(asptLms64f *lms, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	lms64fGetDelayLine(asptLms64f *lms, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
#define	lms64fGetCoefPtr(lms)			((lms)->coef)
#define	lms64fGetFilterLength(lms)		((lms)->filterLength)
#define	lms64fGetDelayLineLength(lms)	((lms)->filterLength)
#define	lms64fGetLastErrorSample(lms)	((lms)->error)
#define	lms64fGetStepSize(lms)			((lms)->stepSize)


/* API to set the properties of an asptLms64f filter */
DT32S	lms64fReset(asptLms64f *lms);
DT32S	lms64fResetDelayLine(asptLms64f *lms);
DT32S	lms64fResize(asptLms64f *lms, DT32S newLength);
DT32S	lms64fSetCoef(asptLms64f *lms, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	lms64fSetDelayLine(asptLms64f *lms, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	lms64fSetStepSize(lms, newStep)	((lms)->stepSize = newStep)

#ifdef __cplusplus
}
#endif

#endif /* ASPTLMS_H_ */
