/*  ********************************************************************
    ASPTLMSLATTICE.H 
    Declares the LMS LATTICE Joint Process Estimator adaptive algorithm.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */

#ifndef ASPTLMSLATTICE_H_
#define ASPTLMSLATTICE_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon64f.h"

typedef struct _asptLmslattice64f {
	DT32S	filterLength;	/* filter length in samples  */
	DT64F	pcStepSize;		/* predictor coef. step size  */
	DT64F	lcStepSize;		/* linear combiner step size  */
	DT64F	error;			/* last error sample */
	DT32S	updateFlag;		/* PARCOR update flag */
	DT64F	*parcor;		/* PARCOR coef. vector */
	DT64F	*coef;			/* linear combiner coef. vector */
	DT64F	*bpe;			/* backward prediction error vector */
	DT64F	*bpePower;		/* power estimate of bpe */
	DT64F	*scratch;		/* working buffer            */
           
} asptLmslattice64f;

/* API to create and delete an asptLmslattice64f filter */
DT32S	lmslattice64fInit(asptLmslattice64f *lmsl, DT32S L, DT64F mu_p, DT64F mu_c);
DT32S	lmslattice64fInitStatic(asptLmslattice64f *lmsl,DT32S L, DT64F mu_p, DT64F mu_c, DT64F *pCof, DT64F *pDat);
void	lmslattice64fFree(asptLmslattice64f *lmsl);

/* APT to filter and update an asptLmslattice64f filter */
DT64F	lmslattice64fFilterUpdate(asptLmslattice64f *lmsl, DT64F x, DT64F d);
DT64F	lmslattice64fFilterOnly(asptLmslattice64f *lmsl, DT64F x);

/* API to retreve the properties of an asptLmslattice64f filter */
DT32S	lmslattice64fGetCoef(asptLmslattice64f *lmsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	lmslattice64fGetParcor(asptLmslattice64f *lmsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	lmslattice64fGetDelayLine(asptLmslattice64f *lmsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
#define	lmslattice64fGetCoefPtr(lmsl)			((lmsl)->coef)
#define	lmslattice64fGetParcorPtr(lmsl)			((lmsl)->parcor)
#define	lmslattice64fGetDelayLinePtr(lmsl)		((lmsl)->bpe)
#define	lmslattice64fGetFilterLength(lmsl)		((lmsl)->filterLength)
#define	lmslattice64fGetLcStepSize(lmsl)		((lmsl)->lcStepSize)
#define	lmslattice64fGetPcStepSize(lmsl)		((lmsl)->pcStepSize)
#define	lmslattice64fGetLastErrorSample(lmsl)	((lmsl)->error)
#define	lmslattice64fGetParcorUpdateFlag(lmsl)	((lmsl)->updateFlag)

/* API to set the properties of an asptLmslattice64f filter */
DT32S	lmslattice64fReset(asptLmslattice64f *lmsl);
DT32S	lmslattice64fResize(asptLmslattice64f *lmsl, DT32S newLength);
DT32S	lmslattice64fSetCoef(asptLmslattice64f *lmsl, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	lmslattice64fSetParcor(asptLmslattice64f *lmsl, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	lmslattice64fSetDelayLine(asptLmslattice64f *lmsl, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	lmslattice64fSetLcStepSize(lmsl, newStep) ((lmsl)->lcStepSize = newStep)
#define	lmslattice64fSetPcStepSize(lmsl, newStep) ((lmsl)->pcStepSize = newStep)
#define	lmslattice64fSetParcorUpdateFlag(lmsl, newFlag) ((lmsl)->updateFlag = newFlag)

#ifdef __cplusplus
}
#endif
#endif /* ASPTLMSLATTICE_H_ */

