/*  ********************************************************************
    ASPTNLMS.H 

    Declares the Normalized LMS adaptive filter algorithm.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTNLMS_H_
#define ASPTNLMS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon64f.h"


typedef struct _asptNlms64f {
	DT32S	filterLength;			/* filter length in samples		*/
	DT64F	stepSize;				/* LMS step size				*/
	DT64F	error;					/* last error sample			*/
	DT64F	power;                  /* last estimated input power	*/
	DT64F	*coef;					/* filter coef. vector			*/
	DT64F	*delayLine;				/* pointer to the filter data	*/
	DT32S	xIndex;
} asptNlms64f;

/* API to create and delete an asptNlms64f filters */
DT32S	nlms64fInit (asptNlms64f *nlms,DT32S L, DT64F mu);
DT32S	nlms64fInitStatic(asptNlms64f *nlms,DT32S L, DT64F mu, DT64F *pCof, DT64F *pDat);
void	nlms64fFree(asptNlms64f *nlms);

/* API to filter through and update an asptNlms64f filters */
DT64F	nlms64fFilterUpdate(asptNlms64f *nlms, DT64F inp, DT64F des);
DT64F	nlms64fFilterOnly  (asptNlms64f *nlms, DT64F inp);
DT64F	nlms64fLCFilterUpdate(asptNlms64f *nlms, DT64F des);
DT64F	nlms64fLCFilterOnly  (asptNlms64f *nlms);

/* API to retrieve the properties of an asptNlms64f filter */
DT32S	nlms64fGetCoef(asptNlms64f *nlms, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	nlms64fGetDelayLine(asptNlms64f *nlms, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
#define	nlms64fGetCoefPtr(nlms)			((nlms)->coef)
#define	nlms64fGetFilterLength(nlms)	((nlms)->filterLength)
#define	nlms64fGetDelayLineLength(nlms)	((nlms)->filterLength)
#define	nlms64fGetLastErrorSample(nlms)	((nlms)->error)
#define	nlms64fGetStepSize(nlms)		((nlms)->stepSize)


/* API to set the properties of an asptNlms64f filter */
DT32S	nlms64fReset(asptNlms64f *nlms);
DT32S	nlms64fResetDelayLine(asptNlms64f *nlms);
DT32S	nlms64fResize(asptNlms64f *nlms, DT32S newLength);
DT32S	nlms64fSetCoef(asptNlms64f *nlms, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	nlms64fSetDelayLine(asptNlms64f *nlms, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	nlms64fSetStepSize(nlms, newStep)	((nlms)->stepSize = newStep)

#ifdef __cplusplus
}
#endif

#endif /* ASPTNLMS_H_ */
