/*  ********************************************************************
    ASPTPBFDAF.C 
    Declares the Partitioned Block Frequency Domain Adaptive Filter (PBFDAF).
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef PBFDAF_H_
#define PBFDAF_H_

#ifdef __cplusplus
extern "C" {
#endif


#include "asptcommon64f.h"
#include "asptfft64f.h"

typedef struct _asptPbfdaf64f {
	DT32S	filterLength;			/* filter length in samples */
	DT32S	blockLength;			/* new samples per block    */
	DT32S	partitions;				/* number of partitions		*/
	DT32S	fftLength;				/* fft length               */	
	DT32S	matrixSize;				/* data matrix size			*/
	DT32S	BO2P1;					/* length of complex vector */
	DT32S	constrain;				/* constrained BFDAF        */
	DT32S	C1;						/* fftLength - blockLength  */
	DT32S	C2;						/* fftLength - filterLength */
	DT64F	B2;						/* power smoothing pole     */
	DT32S	curIndex;				/* index variable			*/
	DT64F	stepSize;				/* BFDAF step size          */
	DT64F	scale;					/* scaling factor           */
	DT64F	*xTime;					/* input vec. in T-domain   */
	DT64F	*xFreq;					/* input vec. in F-domain   */
	DT64F	*wFreq;					/* coef. vector F-domain	*/
	DT64F	*yFreq;					/* filter output            */
	DT64F	*pFreq;					/* input signal power       */
	DT64F	*pNew;					/* new input power          */
	DT64F	*w;						/* FFT twiddle factor table */
	DT32S	*ip;					/* work buffer for FFT      */
} asptPbfdaf64f;

/* API to create and delete asptPbfdaf64f filters */
DT32S	pbfdaf64fInit(asptPbfdaf64f *pbfdaf,DT32S M,DT32S L,DT32S N,DT64F mu,DT32S cs,DT32S *ip,DT64F *w);
DT32S	pbfdaf64fInitStatic(asptPbfdaf64f *pbfdaf,DT32S M,DT32S L,DT32S N,DT64F mu,DT32S cs,DT64F *pCof,DT64F *pDat,DT32S *ip,DT64F *w);
void	pbfdaf64fFree(asptPbfdaf64f *pbfdaf);


/* API to filter and update asptPbfdaf64f filters */
DT32S pbfdaf64fFilterUpdate(asptPbfdaf64f *pbfdaf, DT64F *inp, DT64F *des, DT64F *out, DT64F *err);
DT32S pbfdaf64fFilterOnly(asptPbfdaf64f *pbfdaf, DT64F *inp, DT64F *out);

/* APT to retrieve the properties of asptPbfdaf64f filters */
DT32S	pbfdaf64fGetCoef(asptPbfdaf64f *pbfdaf, DT64F *dstBuf, DT32S ind, DT32S N, DT32S flip);
#define	pbfdaf64fGetCoefPtr(pbfdaf) 		((pbfdaf)->wFreq)
#define	pbfdaf64fGetFilterLength(pbfdaf) 	((pbfdaf)->filterLength)
#define	pbfdaf64fGetBlockLength(pbfdaf) 	((pbfdaf)->blockLength)
#define	pbfdaf64fGetFftLength(pbfdaf) 		((pbfdaf)->fftLength)
#define	pbfdaf64fGetStepSize(pbfdaf) 		((pbfdaf)->stepSize)
#define	pbfdaf64fGetConstrain(pbfdaf) 		((pbfdaf)->constrain)
#define	pbfdaf64fGetPartitions(pbfdaf)		((pbfdaf)->partitions)

/* API to set the properties of asptPbfdaf64f filters */
DT32S	pbfdaf64fReset(asptPbfdaf64f *pbfdaf);
DT32S	pbfdaf64fResetDelayLine(asptPbfdaf64f *pbfdaf);
DT32S	pbfdaf64fResize(asptPbfdaf64f *pbfdaf, DT32S newLen, DT32S newBlock, DT32S newPart);
DT32S	pbfdaf64fSetCoef(asptPbfdaf64f *pbfdaf, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	pbfdaf64fSetStepSize(pbfdaf, newSs) 	((pbfdaf)->stepSize = newSs)
#define	pbfdaf64fSetConstrain(pbfdaf, newC) 	((pbfdaf)->constrain = newC)


#ifdef __cplusplus
}
#endif

#endif /* PBFDAF_H_ */
