/*  ********************************************************************
    ASPTRLS.H 

    Declares the Recursive Least Squares (RLS) adaptive filter algorithm.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */

#ifndef ASPTRLS_H_
#define ASPTRLS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon64f.h"

#define ZUINIG

typedef struct _asptRls64f {
	DT32S		filterLength;	/* filter length in samples */
#ifndef ZUINIG
	DT32S		lenSquare;		/* filterLength ^ 2         */
#endif
	DT64F	forgetFactor;	/* forgetting factor        */
	DT64F	recForgetFactor;/* 1/(forgetFactor)         */
	DT64F	error;			/* last error sample		*/
	DT64F	*coef;			/* filter coef. vector		*/
	DT64F	*delayLine;		/* delay line vector		*/
	DT64F	*acor;			/* auto-corr. matrix inverse*/
	DT64F	*scratch;		/* working buffer           */

} asptRls64f;

/* API to create and delete an asptRls64f filters */
DT32S	rls64fInit (asptRls64f *rls, DT32S L, DT64F mu, DT64F b);
DT32S	rls64fInitStatic(asptRls64f *rls, DT32S L, DT64F mu, DT64F b, DT64F *pCof, DT64F *pDat);
void	rls64fFree(asptRls64f *rls);

/* API to filter through and update an asptRls64f filters */
DT64F	rls64fFilterUpdate(asptRls64f *rls, DT64F inp, DT64F des);
DT64F	rls64fFilterOnly(asptRls64f *rls, DT64F inp);
DT64F	rls64fLCFilterUpdate(asptRls64f *rls, DT64F des);
DT64F	rls64fLCFilterOnly(asptRls64f *rls);

/* API to retrieve the properties of an asptRls64f objects */
DT32S	rls64fGetCoef(asptRls64f *rls, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rls64fGetDelayLine(asptRls64f *rls, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
#define	rls64fGetCoefPtr(rls)			((rls)->coef)
#define	rls64fGetFilterLength(rls)		((rls)->filterLength)
#define	rls64fGetDelayLineLength(rls)	((rls)->filterLength)
#define	rls64fGetLastErrorSample(rls)	((rls)->error)
#define	rls64fGetForgetFactor(rls)		((rls)->forgetFactor)

/* API to set the properties of an asptRls64f filter */
DT32S	rls64fReset(asptRls64f *rls);
DT32S	rls64fResetDelayLine(asptRls64f *rls);
DT32S	rls64fResize(asptRls64f *rls, DT32S newLength);
DT32S	rls64fSetCoef(asptRls64f *rls, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	rls64fSetDelayLine(asptRls64f *rls, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	rls64fSetForgetFactor(rls, newFactor) ((rls)->forgetFactor = newFactor)


#ifdef __cplusplus
}
#endif
#endif /* ASPTRLS_H_ */

