/*  ********************************************************************
    ASPTRLSLATTICE.H 

    Declares the Recursive Least Squares Lattice joint process estimator
    using the a posteriori estimation errors.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTRLSLATTICE_H_
#define ASPTRLSLATTICE_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon64f.h"

typedef struct _asptRlslattice64f {
	DT64F	*ff;			/* autocorrelation vector of forward prediction error  */
	DT64F	*bb;			/* autocorrelation vector of backward prediction error  */
	DT64F	*fb;			/* cross-correlation of f and b  */
	DT64F	*be;			/* cross-correlation of b and e  */
	DT64F	*cf;			/* conversion factor */
	DT64F	*b;				/* vector of backward prediction error  */
	DT64F	*kf;			/* forward lattice coefficients  */
	DT64F	*kb;			/* backward lattice coefficients */
	DT64F	*c;				/* linear combiner coefficients  */
	DT32S	filterLength;	/* linear combiner length in samples  */
	DT64F	forgetFactor;	/* forgetting factor  */
	DT64F	error;			/* last error sample           */
} asptRlslattice64f;

/* API to create and delete an asptRlslattice64f filter */
DT32S	rlslattice64fInit(asptRlslattice64f *rlsl, DT32S L, DT64F mu);
DT32S	rlslattice64fInitStatic(asptRlslattice64f *rlsl, DT32S L, DT64F mu, DT64F *pCof, DT64F *pDat);
void	rlslattice64fFree(asptRlslattice64f *rlsl);

/* APT to filter and update an asptRlslattice64f filter */
DT64F	rlslattice64fFilterUpdate(asptRlslattice64f *rlsl, DT64F x, DT64F d);
DT64F	rlslattice64fFilterOnly(asptRlslattice64f *rlsl, DT64F x);

/* API to retreve the properties of an asptRlslattice64f filter */
DT32S	rlslattice64fGetCoef(asptRlslattice64f *rlsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslattice64fGetForwardParcors(asptRlslattice64f *rlsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslattice64fGetBackwardParcors(asptRlslattice64f *rlsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslattice64fGetDelayLine(asptRlslattice64f *rlsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
#define	rlslattice64fGetCoefPtr(rlsl)			((rlsl)->c)
#define	rlslattice64fGetForwardParcorPtr(rlsl)	((rlsl)->kf)
#define	rlslattice64fGetBackwardParcorPtr(rlsl)	((rlsl)->kb)
#define	rlslattice64fGetDelayLinePtr(rlsl)		((rlsl)->b)
#define	rlslattice64fGetFilterLength(rlsl)		((rlsl)->filterLength)
#define	rlslattice64fGetForgetFactor(rlsl)		((rlsl)->forgetFactor)
#define	rlslattice64fGetLastErrorSample(rlsl)	((rlsl)->error)

/* API to set the properties of an asptRlslattice64f filter */
DT32S	rlslattice64fReset(asptRlslattice64f *rlsl);
DT32S	rlslattice64fResize(asptRlslattice64f *rlsl, DT32S newLength);
DT32S	rlslattice64fSetDelayLine(asptRlslattice64f *rlsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
#define	rlslattice64fSetForgetFactor(rlsl, newFactor) ((rlsl)->forgetFactor = newFactor)

#ifdef __cplusplus
}
#endif
#endif /* ASPTRLSLATTICE_H_ */

