/*  ********************************************************************
    ASPTRLSLBPEF.H 

    Declares Recursive Least Squares Lattice Backward Prediction Error Filter
    using the a posteriori estimation errors.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTRLSLBPEF_H_
#define ASPTRLSLBPEF_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon64f.h"

typedef struct _asptRlslbpef64f {
	DT64F	*ff;			/* autocorrelation vector of forward prediction error  */
	DT64F	*bb;			/* autocorrelation vector of backward prediction error  */
	DT64F	*fb;			/* cross-correlation of f and b  */
/*	DT64F	*be;			// cross-correlation of b and e  */
	DT64F	*cf;			/* conversion factor */
	DT64F	*b;				/* vector of backward prediction error  */
	DT64F	*kf;			/* forward lattice coefficients  */
	DT64F	*kb;			/* backward lattice coefficients */
	DT64F	*c;				/* linear combiner coefficients  */
	DT64F	*inBuf;			/* input signal buffer */
	DT32S		index;			/* index of last inserted input sample */
	DT32S	filterLength;	/* linear combiner length in samples  */
	DT64F	forgetFactor;	/* forgetting factor  */
	DT64F	error;			/* last error sample   */
	DT32S		firFlag;		/* if non-zero will calculate the equivalent transversal */
							/* coefficients and store them in c each iteration. */
} asptRlslbpef64f;

/* API to create and delete an asptRlslbpef64f filter */
DT32S	rlslbpef64fInit(asptRlslbpef64f *rlsl, DT32S L, DT64F mu);
DT32S	rlslbpef64fInitStatic(asptRlslbpef64f *rlsl, DT32S L, DT64F mu, DT64F *pCof, DT64F *pDat);
void	rlslbpef64fFree(asptRlslbpef64f *rlsl);

/* APT to filter and update an asptRlslbpef64f filter */
DT64F	rlslbpef64fFilterUpdate(asptRlslbpef64f *rlsl, DT64F x);
DT64F	rlslbpef64fFilterOnly(asptRlslbpef64f *rlsl, DT64F x);

/* API to retreve the properties of an asptRlslbpef64f filter */
DT32S	rlslbpef64fGetCoef(asptRlslbpef64f *rlsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslbpef64fGetForwardParcors(asptRlslbpef64f *rlsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslbpef64fGetBackwardParcors(asptRlslbpef64f *rlsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslbpef64fGetDelayLine(asptRlslbpef64f *rlsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
#define	rlslbpef64fGetCoefPtr(rlsl)				((rlsl)->c)
#define	rlslbpef64fGetForwardParcorPtr(rlsl)	((rlsl)->kf)
#define	rlslbpef64fGetBackwardParcorPtr(rlsl)	((rlsl)->kb)
#define	rlslbpef64fGetDelayLinePtr(rlsl)		((rlsl)->b)
#define	rlslbpef64fGetFilterLength(rlsl)		((rlsl)->filterLength)
#define	rlslbpef64fGetForgetFactor(rlsl)		((rlsl)->forgetFactor)
#define	rlslbpef64fGetLastErrorSample(rlsl)		((rlsl)->error)
#define	rlslbpef64fGetFirFlag(rlsl)				((rlsl)->firFlag)

/* API to set the properties of an asptRlslbpef64f filter */
DT32S	rlslbpef64fReset(asptRlslbpef64f *rlsl);
DT32S	rlslbpef64fResize(asptRlslbpef64f *rlsl, DT32S newLength);
DT32S	rlslbpef64fSetDelayLine(asptRlslbpef64f *rlsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
#define	rlslbpef64fSetForgetFactor(rlsl, newFactor) ((rlsl)->forgetFactor = newFactor)
#define	rlslbpef64fSetFirFlag(rlsl, newFlag)		((rlsl)->firFlag = newFlag)

#ifdef __cplusplus
}
#endif
#endif /* ASPTRLSLBPEF_H_ */

