/*  ********************************************************************
    ASPTRLSLFPEF.H 

    Declares Recursive Least Squares Lattice Forward Prediction Error 
	Filter using the a posteriori estimation errors.
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef ASPTRLSLFPEF_H_
#define ASPTRLSLFPEF_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon64f.h"

typedef struct _asptRlslfpef64f {
	DT64F	*ff;			/* autocorrelation vector of forward prediction error  */
	DT64F	*bb;			/* autocorrelation vector of backward prediction error  */
	DT64F	*fb;			/* cross-correlation of f and b  */
/*	DT64F	*be;			// cross-correlation of b and e  */
	DT64F	*cf;			/* conversion factor */
	DT64F	*b;				/* vector of backward prediction error  */
	DT64F	*kf;			/* forward lattice coefficients  */
	DT64F	*kb;			/* backward lattice coefficients */
	DT64F	*c;				/* linear combiner coefficients  */
	DT32S	filterLength;	/* linear combiner length in samples  */
	DT64F	forgetFactor;	/* forgetting factor  */
	DT64F	error;			/* last error sample   */
	DT32S	firFlag;		/* if non-zero will calculate the equivalent transversal */
							/* coefficients and store them in c each iteration. */
} asptRlslfpef64f;

/* API to create and delete an asptRlslfpef64f filter */
DT32S	rlslfpef64fInit(asptRlslfpef64f *rlsl, DT32S L, DT64F mu);
DT32S	rlslfpef64fInitStatic(asptRlslfpef64f *rlsl, DT32S L, DT64F mu, DT64F *pCof, DT64F *pDat);
void	rlslfpef64fFree(asptRlslfpef64f *rlsl);

/* APT to filter and update an asptRlslfpef64f filter */
DT64F	rlslfpef64fFilterUpdate(asptRlslfpef64f *rlsl, DT64F x);
DT64F	rlslfpef64fFilterOnly(asptRlslfpef64f *rlsl, DT64F x);

/* API to retreve the properties of an asptRlslfpef64f filter */
DT32S	rlslfpef64fGetCoef(asptRlslfpef64f *rlsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslfpef64fGetForwardParcors(asptRlslfpef64f *rlsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslfpef64fGetBackwardParcors(asptRlslfpef64f *rlsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
DT32S	rlslfpef64fGetDelayLine(asptRlslfpef64f *rlsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
#define	rlslfpef64fGetCoefPtr(rlsl)				((rlsl)->c)
#define	rlslfpef64fGetForwardParcorPtr(rlsl)	((rlsl)->kf)
#define	rlslfpef64fGetBackwardParcorPtr(rlsl)	((rlsl)->kb)
#define	rlslfpef64fGetDelayLinePtr(rlsl)		((rlsl)->b)
#define	rlslfpef64fGetFilterLength(rlsl)		((rlsl)->filterLength)
#define	rlslfpef64fGetForgetFactor(rlsl)		((rlsl)->forgetFactor)
#define	rlslfpef64fGetLastErrorSample(rlsl)		((rlsl)->error)
#define	rlslfpef64fGetFirFlag(rlsl)				((rlsl)->firFlag)

/* API to set the properties of an asptRlslfpef64f filter */
DT32S	rlslfpef64fReset(asptRlslfpef64f *rlsl);
DT32S	rlslfpef64fResize(asptRlslfpef64f *rlsl, DT32S newLength);
DT32S	rlslfpef64fSetDelayLine(asptRlslfpef64f *rlsl, DT64F *buf, DT32S ind, DT32S N, DT32S flip);
#define	rlslfpef64fSetForgetFactor(rlsl, newFactor) ((rlsl)->forgetFactor = newFactor)
#define	rlslfpef64fSetFirFlag(rlsl, newFlag)		((rlsl)->firFlag = newFlag)

#ifdef __cplusplus
}
#endif
#endif /* ASPTRLSLFPEF_H_ */

