/*  ********************************************************************
    ASPTTDLMS.C 
    Declares the Transform-Domain LMS Adaptive Filter (TDLMS).
    ******************************************************************** */

/*  *************  Version 2.1, Release October 2002. ****************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#ifndef TDLMS_H_
#define TDLMS_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "asptcommon64f.h"
#include "asptfft64f.h"

typedef struct _asptTdlms64f {
	DT32S	filterLength;			/* filter length in samples */
	DT32S	fftLength;				/* fft length               */	
	DT32S	BO2P1;					/* length of complex vector */
	DT64F	stepSize;				/* TDLMS step size          */
	DT64F	error;					/* last error sample        */
	DT64F	scale;					/* scaling factor           */
	DT32S	idx;
	DT64F	*xTime;					/* input vec. in T-domain   */
	DT64F	*xFreq;					/* input vec. in F-domain   */
	DT64F	*wFreq;					/* coef. vector F-domain	*/
	DT64F	*pFreq;					/* input signal power       */
	DT64F	*pNew;					/* new input power          */
	DT64F	*w;						/* FFT twiddle factor table */
	DT32S	*ip;					/* work buffer for FFT      */
} asptTdlms64f;

/* API to create and delete asptTdlms64f filters */
DT32S	tdlms64fInit(asptTdlms64f *tdlms,DT32S L,DT64F mu,DT32S *ip,DT64F *w);
DT32S	tdlms64fInitStatic(asptTdlms64f *tdlms,DT32S L,DT64F mu,DT64F *pCof,DT64F *pDat,DT32S *ip,DT64F *w);
void	tdlms64fFree(asptTdlms64f *tdlms);


/* API to filter and update asptTdlms64f filters */
DT64F tdlms64fFilterUpdate(asptTdlms64f *tdlms, DT64F inp, DT64F des);
DT64F tdlms64fFilterOnly(asptTdlms64f *tdlms, DT64F inp);
DT64F tdlms64fLCFilterUpdate(asptTdlms64f *tdlms, DT64F des);
DT64F tdlms64fLCFilterOnly(asptTdlms64f *tdlms);

/* APT to retrieve the properties of asptTdlms64f filters */
DT32S	tdlms64fGetCoef(asptTdlms64f *tdlms, DT64F *dstBuf, DT32S ind, DT32S N, DT32S flip);
DT32S	tdlms64fGetDelayLine(asptTdlms64f *tdlms, DT64F *dstBuf, DT32S ind, DT32S N, DT32S flip);
#define	tdlms64fGetCoefPtr(tdlms) 			((tdlms)->wFreq)
#define	tdlms64fGetFilterLength(tdlms) 		((tdlms)->filterLength)
#define	tdlms64fGetFftLength(tdlms) 		((tdlms)->fftLength)
#define	tdlms64fGetDelayLineLength(tdlms) 	((tdlms)->fftLength)
#define	tdlms64fGetStepSize(tdlms) 			((tdlms)->stepSize)
#define	tdlms64fGetLastErrorSample(tdlms) 	((tdlms)->error)

/* API to set the properties of asptTdlms64f filters */
DT32S	tdlms64fReset(asptTdlms64f *tdlms);
DT32S	tdlms64fResetDelayLine(asptTdlms64f *tdlms);
DT32S	tdlms64fResize(asptTdlms64f *tdlms, DT32S newLen);
DT32S	tdlms64fSetCoef(asptTdlms64f *tdlms, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
DT32S	tdlms64fSetDelayLine(asptTdlms64f *tdlms, DT64F *newCoef, DT32S ind, DT32S N, DT32S flip);
#define	tdlms64fSetStepSize(tdlms, newSs) 	((tdlms)->stepSize = newSs)


#ifdef __cplusplus
}
#endif

#endif /* TDLMS_H_ */
