/*  ********************************************************************
    TESTADJLMS.C
    Test application for ADJLMS
    ******************************************************************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */
#include <time.h>
#include "asptadjlms32f.h"
#include "asptfir32f.h"
#include "asptdisplay32f.h"


/****************/
/* Test Options */
/****************/
//#define STATIC
//#define RESIZE
//#define RESET
//#define FILTERONLY
//#define DELAYLINE
#define PRINTCOEF
//#define MACROS

#define ITERATIONS 5000

int main(int argc, char* argv[])
{

	asptAdjlms32f  adjlms ;
	int		len = 10;					/* controller length	       */
	asptFir32f pFir;					/* FIR filter for prim signal  */
	float	mu  = 0.01f ;				/* step size			       */
	int		i,j,k=0;
	FILE	*senFile;					/* sensor signal log file      */
	FILE	*actFile;					/* sec. source signal log file */
	float	ph[] = {0.0f,1.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f};	/* Primary path impulse response	*/
	float	sh[] = {-1.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f};/* Secondary path impulse response	*/
	float	se[10];						/* estimation of sh	*/
	float	reference[ITERATIONS];		/* input reference signal					*/
	float	primary[ITERATIONS];		/* primary source signal					*/
	float	actBuf[ITERATIONS];		/* secondary source signal at the sensor	*/
	float	senBuf[ITERATIONS];		/* secondary source signal at the sensor	*/
	int		phLen = 10;
	int		shLen = 10;
	int		sheLen= 7;
	int		idx = 0;
	char	*senFileName="sen.txt";
	char	*actFileName="sec.txt";
	char	*firFileName="coef.txt";
   clock_t	start, finish;
   double	duration;

#ifdef PRINTCOEF
	FILE	*firFile;					/* file pointer for dumping FIR coef.*/
	float	*coef;
	float	fir[20];					/* buffer to print the controller coef.*/
#endif

#ifdef DELAYLINE
#ifndef PRINTCOEF
	float	fir[20];					/* buffer to print the delay line samples */
#endif
#endif

#ifdef STATIC
	float	pDat[10+10+2*7-1];
	float	pCof[10];
#endif

	/* make a random input signal */
	for (i=0; i<ITERATIONS; i++)
		reference[i] = (float) rand()/(float)RAND_MAX - 0.5f;

	j = MIN(shLen, sheLen);
	/* Estimate the secondary path */
	for (i=0; i<j; i++)	{
		se[i] = 1.0f * sh[i];
	}

	/* reset the buffers; */
	reset32f(actBuf, ITERATIONS);
	reset32f(senBuf, ITERATIONS);

	i = fir32fInit(&pFir, phLen, ph);

	/* calculate the primary signal */
	fir32fBlockFilter(&pFir, reference, primary, ITERATIONS);

	/* Create and initialize the ADJLMS */
#ifndef STATIC
		i = adjlms32fInit(&adjlms,len,mu,sh,shLen,se,sheLen);
#else
		i = adjlms32fInitStatic(&adjlms,len,mu,sh,shLen,se,sheLen,pCof,pDat);
#endif
		
	if (adjlms32fGetFilterLength(&adjlms) == 0){
		printf("ERROR %d : Error creating ADJLMS filter\n",i);
		exit(-1);
	}

#ifdef FILTERONLY
	i = adjlms32fSetCoef(&adjlms, ph, 0, len, 0);
	printf("coef copied = %d\n",i);
	dispR32f(adjlms.coef, i);
#endif

	start = clock();
	duration = (double) 0.0;

	for (i=0; i<ITERATIONS;i++)	{
		
#ifdef FILTERONLY
		if (i < 1500){
			/* filter only */
			actBuf[i] = adjlms32fFilterOnly(&adjlms, reference[i]);
			senBuf[i]  = primary[i] + actBuf[i];
		}
		else
#endif
		{
			/* filter and update the filter coefficients */
			actBuf[i] = adjlms32fFilterUpdate(&adjlms, reference[i], primary[i]);
			senBuf[i]  = adjlms32fGetLastSensorSample(&adjlms);
		}

#ifdef RESET
		if (i == 2000){
			/* reset */
			adjlms32fReset(&adjlms);
		}
#endif

#ifdef RESIZE
#ifndef STATIC
		if (i == 3000){
			/* resize */
			j = adjlms32fResize(&adjlms, 11);
			if (j != 0) {
				printf("Couldn't resize\n");
				exit(-1);
			}
			dispR32f(adjlms.coef, adjlms32fGetFilterLength(&adjlms));
		}
#endif
#endif
	} 

	finish = clock();
	duration = (double)(finish - start) ;

	/* Open files */
	actFile	= fopen (actFileName, "wt") ;
	if (actFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing",actFileName); 
		return(-2);
	}

	senFile	= fopen (senFileName, "wt") ;
	if (senFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", senFileName); 
		return(-3);
	}
	
	/* write output and error samples to streams */
	for (i=0; i<ITERATIONS;i++)	{
		fprintf(senFile, "%f\n", senBuf[i]);
		fprintf(actFile, "%f\n", actBuf[i]);
	}
	fclose(senFile);
	fclose(actFile);

#ifdef PRINTCOEF
	/* save and print the filter coefficients */
	
	firFile	= fopen (firFileName, "wt") ;
	if (firFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", firFileName); 
		return(-4);
	}

	coef = adjlms32fGetCoefPtr(&adjlms);
	i = adjlms32fGetCoef(&adjlms, fir, 0, len, 0);

	dispR32f(coef, adjlms32fGetFilterLength(&adjlms));
	//fdispR32f(firFile, fir, adjlms32fGetFilterLength(&adjlms));

	for (i=0; i<len; i++)
		fprintf(firFile, "%f  ", fir[i]);

	fclose(firFile);
#endif


#ifdef DELAYLINE
	/* test delay line Set and Get */
	adjlms32fGetDelayLine(&adjlms, fir, 0, 10, 0);
	dispR32f(fir, adjlms32fGetFilterLength(&adjlms));
//	adjlms32fResetDelayLine(&adjlms);
	adjlms32fGetDelayLine(&adjlms, fir, 0, 10, 0);
	dispR32f(fir, adjlms32fGetFilterLength(&adjlms));

	i = adjlms32fSetDelayLine(&adjlms, ph, 0, 5, 0);

	i = adjlms32fGetDelayLine(&adjlms, fir, 0, 10, 0);
	dispR32f(fir, adjlms32fGetFilterLength(&adjlms));

#endif

#ifdef MACROS
	printf("FilterLength= %d, stepSize= = %f, coefPtr=%d, secPtr=%d, estSecPtr=%d, secLength=%d, estSecLength=%d\n", 
		adjlms32fGetFilterLength(&adjlms),  
		adjlms32fGetStepSize(&adjlms),
		adjlms32fGetCoefPtr(&adjlms),
		adjlms32fGetSecPathPtr(&adjlms),
		adjlms32fGetEstSecPathPtr(&adjlms),
		adjlms32fGetSecPathLength(&adjlms),
		adjlms32fGetEstSecPathLength(&adjlms) );

	adjlms32fSetStepSize(&adjlms, 0.111f);
	adjlms32fSetSecPathPtr(&adjlms, NULL);
	adjlms32fSetEstSecPathPtr(&adjlms, NULL);
	adjlms32fSetSecPathLength(&adjlms, 0);
	adjlms32fSetEstSecPathLength(&adjlms, 0);

	printf("FilterLength= %d, stepSize= = %f, coefPtr=%d, secPtr=%d, estSecPtr=%d, secLength=%d, estSecLength=%d\n", 
		adjlms32fGetFilterLength(&adjlms),  
		adjlms32fGetStepSize(&adjlms),
		adjlms32fGetCoefPtr(&adjlms),
		adjlms32fGetSecPathPtr(&adjlms),
		adjlms32fGetEstSecPathPtr(&adjlms),
		adjlms32fGetSecPathLength(&adjlms),
		adjlms32fGetEstSecPathLength(&adjlms) );
#endif

  printf(  "Filter Length = %d\n",  adjlms32fGetFilterLength(&adjlms));
#ifndef STATIC
	adjlms32fFree(&adjlms);
#endif

  printf("\nElapsed Time    in Seconds\n");
  printf(  "ADJLMS          %10.10f\n",  (float)duration/ CLOCKS_PER_SEC );

	return 0;
}


