/*  ********************************************************************
	TESTBFDAF.C
	Test application for the BFDAF filter
    ******************************************************************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#include <time.h>
#include "asptbfdaf32f.h"
#include "asptfir32f.h"
#include "asptdisplay32f.h"
#include "rfft16.h"
//#include "rfft32.h"

/****************/
/* Test Options */
/****************/

//#define STATIC
//#define RESIZE
//#define RESET
//#define FILTERONLY
#define PRINTCOEF
//#define MACROS
//#define SPEED

#define ITERATIONS 5000
#define NC 10		/* controller length     */
#define NL 6		/* block length          */
#define FL 16       /* FFT length            */
#define HL 8        /* unknown filter length */

int main(int argc, char* argv[])
{

	int		i,j,k;
	asptBfdaf32f  bfdaf ;		/* adaptive filter          */
	asptFir32f hFir;			/* unknown filter           */
	int		Nc = NC;			/* filter length		    */
	int		Nl = NL;			/* block length             */
	int		cs = 1;				/* use constrained BFDAF    */
	float	mu = 0.05f;			/* step size                */

	FILE	*outFile;			/* output signal log file	*/
	FILE	*errFile;			/* error signal log file	*/
	FILE	*inpFile;			/* input signal log file	*/
	FILE	*desFile;			/* desired signal log file	*/

	float	inpBuf[2*NL];
	float	desBuf[2*NL];
	float	errBuf[2*NL];
	float	outBuf[2*NL];
	float	unknown[]={.1f, .3f, -.4f, .7f, -.3f, .9f, .5f, .2f};

	float	fir[2*NC];			/* buffer for printing purposes */

	char	*outFileName="out.txt";
	char	*errFileName="err.txt";
	char	*inpFileName="inp.txt";
	char	*desFileName="des.txt";

	int		*ip = ip16;			/* twiddle factor table          */
	float	*w  = w16;          /* bit reversal table            */
	clock_t start, finish;
	double  duration;

#ifdef PRINTCOEF
	FILE	*firFile;			/* FIR coef. file           */	
	char	*firFileName="cof.txt";
#endif
	
#ifdef STATIC
	float	pCoef[FL];						/* [fftLength]                          */
	float	pDat[(FL-NL)+2*FL+2*((FL>>1)+1)]; /* [C1+2*fftLength+2*(fftLength/2+1)] */	
#endif


	/* create the unknown filter */
	i = fir32fInit(&hFir, HL, unknown);
	if (fir32fGetFilterLength(&hFir)==0){
		printf("Couldn't create unknown filter\n");
		exit(-1);
	}

	/* reset the buffers; */
	reset32f(inpBuf, 2*NL);
	reset32f(desBuf, 2*NL);
	reset32f(outBuf, 2*NL);
	reset32f(errBuf, 2*NL);	

	/* Create and initialize the asptBfdaf filter */
#ifndef STATIC
		i = bfdaf32fInit(&bfdaf,Nc,Nl,mu,cs, ip,w);
#else
		i = bfdaf32fInitStatic(&bfdaf,Nc,Nl,mu,cs,pCoef,pDat,ip,w);
#endif	
	
	if (i < 0){
		printf("Error creating asptBfdaf filter\n");
		exit(-1);
	}
/*
	printf("FilterLength = %d\n", bfdaf.filterLength);
	printf("BlockLength  = %d\n", bfdaf.blockLength);
	printf("FFTLength    = %d\n", bfdaf.fftLength);
	printf("C1           = %d\n", bfdaf.C1);
*/

#ifdef FILTERONLY
	j= bfdaf32fSetCoef(&bfdaf, unknown, 0, NC,0);
	printf("Coef set = %d\n",j);
	bfdaf32fGetCoef(&bfdaf, fir, 0, NC, 0);
	dispR32f(fir,j);	
#endif

	/* Open files */
	inpFile	= fopen (inpFileName, "wt") ;
	if (inpFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing",inpFileName); 
		return(-2);
	}

	desFile	= fopen (desFileName, "wt") ;
	if (desFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", desFileName); 
		return(-3);
	}

	outFile	= fopen (outFileName, "wt") ;
	if (outFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", outFileName); 
		return(-3);
	}
	
	errFile	= fopen (errFileName, "wt") ;
	if (errFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", errFileName); 
		return(-3);
	}

	start = clock();
	duration = (double) 0.0;

	for (i=0; i< (ITERATIONS-Nl);i+=Nl)	{

		/* fill in the input block */
		for (k=0; k<Nl; k++){		
			inpBuf[k] = (float) rand()/(float)RAND_MAX - 0.5f;;	
		}

		/* desired signal */
		fir32fBlockFilter(&hFir, inpBuf, desBuf, Nl);


#ifdef FILTERONLY
		if (i < 400*Nl)	{	
			/* filter but no update */
			bfdaf32fFilterOnly(&bfdaf, inpBuf, outBuf);
			sub32f(outBuf, desBuf, errBuf, Nl);
		}
		else
#endif
		{
			/* filter and update the filter coefficients */
			bfdaf32fFilterUpdate(&bfdaf, inpBuf, desBuf, outBuf,errBuf);
		}
#ifndef SPEED
		/* write samples to streams */
		for (j=0; j<Nl;j++)	{
			fprintf(inpFile, "%+6.5f\n", inpBuf[j]);
			fprintf(desFile, "%+6.5f\n", desBuf[j]);
			fprintf(outFile, "%+6.5f\n", outBuf[j]);		
			fprintf(errFile, "%+6.5f\n", errBuf[j]);
		}
#endif

#ifdef RESET
		if (i == 500*Nl)	{
			bfdaf32fReset(&bfdaf);
		}
#endif

#ifdef RESIZE
#ifndef STATIC
		if (i == 500 * Nl){
			Nc = 8;
			Nl = 7;
			if (Nl<NL)
			   i+= (NL-Nl);
			j = bfdaf32fResize(&bfdaf, Nc,Nl);
			printf("RESIZE returned %d\n",j);
			j = bfdaf32fGetFilterLength(&bfdaf);
			k = bfdaf32fGetCoef(&bfdaf, fir, 0, j, 0);
			dispR32f(fir, k);
		}

#endif
#endif
	} 

		
	finish = clock();
	duration = (double)(finish - start) ;

	fclose(inpFile);
	fclose(desFile);
	fclose(errFile);
	fclose(outFile);

#ifdef PRINTCOEF
	/* save and print the filter coefficients */
	firFile	= fopen (firFileName, "wt") ;
	if (firFile==NULL){
		fprintf(stdout,"Can't open file %s for writing", firFileName); 
		return(-4);
	}

	//copy32f(bfdaf.wFreq,fir,FL);
	//dispR32f(fir, FL);
	bfdaf32fGetCoef(&bfdaf, fir,0, Nc,0);
	dispR32f(fir, bfdaf32fGetFilterLength(&bfdaf));
	fdispC32f(firFile, fir, bfdaf32fGetFilterLength(&bfdaf));

	fclose(firFile);
#endif

#ifdef MACROS
	printf("FL=%d, BL=%d, FFTL=%d, SS=%f, CS=%d, coefPtr=%d,\n", 
	bfdaf32fGetFilterLength(&bfdaf),  
	bfdaf32fGetBlockLength(&bfdaf), 
	bfdaf32fGetFftLength(&bfdaf),
	bfdaf32fGetStepSize(&bfdaf),
	bfdaf32fGetConstrain(&bfdaf), 
	bfdaf32fGetCoefPtr(&bfdaf));

#ifndef STATIC
	bfdaf32fResize(&bfdaf,12,4);
#endif

	bfdaf32fSetStepSize(&bfdaf, 0.111f);
	bfdaf32fSetConstrain(&bfdaf, 0);

	printf("FL=%d, BL=%d, FFTL=%d, SS=%f, CS=%d, coefPtr=%d,\n", 
	bfdaf32fGetFilterLength(&bfdaf),  
	bfdaf32fGetBlockLength(&bfdaf), 
	bfdaf32fGetFftLength(&bfdaf),
	bfdaf32fGetStepSize(&bfdaf),
	bfdaf32fGetConstrain(&bfdaf), 
	bfdaf32fGetCoefPtr(&bfdaf));
#endif


#ifndef STATIC
	bfdaf32fFree(&bfdaf);
#endif

   printf("\nElapsed Time      in Seconds\n");
   printf(  "BFDAF             %10.10f\n",  (float)duration/ CLOCKS_PER_SEC );


	return 0;
}



