/*  ********************************************************************
    TESTFXLMS.C
    Test application for FXLMS
    ******************************************************************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */
#include <time.h>
#include "asptfxlms32f.h"
#include "asptfir32f.h"
#include "asptdisplay32f.h"


/****************/
/* Test Options */
/****************/
//#define STATIC
//#define RESIZE
//#define RESET
//#define FILTERONLY
//#define DELAYLINE
#define PRINTCOEF
//#define MACROS

#define ITERATIONS 5000

int main(int argc, char* argv[])
{

	asptFxlms32f  fxlms ;
	int		len = 10;					/* controller length	       */
	asptFir32f pFir;					/* FIR filter for prim signal  */
	float	mu  = 0.01f ;				/* step size			       */
	int		i,j,k=0;
	FILE	*senFile;					/* sensor signal log file      */
	FILE	*actFile;					/* sec. source signal log file */
	FILE	*primFile;					/* primary signal log file     */
	float	ph[] = {0.0f,1.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f};	/* Primary path impulse response	*/
	float	sh[] = {-1.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f};/* Secondary path impulse response	*/
	float	se[10];						/* estimation of sh	*/
	float	reference[ITERATIONS];		/* input reference signal					*/
	float	primary[ITERATIONS];		/* primary source signal					*/
	float	actBuf[ITERATIONS];		/* secondary source signal at the sensor	*/
	float	senBuf[ITERATIONS];		/* secondary source signal at the sensor	*/
	int		phLen = 10;
	int		shLen = 10;
	int		sheLen= 7;
	int		idx = 0;
	char	*senFileName="sen.txt";
	char	*actFileName="sec.txt";
	char	*firFileName="coef.txt";
	char	*primFileName="prim.txt";
   clock_t	start, finish;
   double	duration;

#ifdef PRINTCOEF
	FILE	*firFile;					/* file pointer for dumping FIR coef.*/
	float	*coef;
	float	fir[20];					/* buffer to print the controller coef.*/
#endif

#ifdef DELAYLINE
#ifndef PRINTCOEF
	float	fir[20];					/* buffer to print the delay line samples */
#endif
#endif

#ifdef STATIC
	/* MAX(L,seL)+sL+L */
	float	pDat[10+10+10];
	float	pCof[10];
#endif

	/* make a random input signal */
	for (i=0; i<ITERATIONS; i++)
		reference[i] = (float) rand()/(float)RAND_MAX - 0.5f;

	j = MIN(shLen, sheLen);
	/* Estimate the secondary path */
	for (i=0; i<j; i++)	{
		se[i] = 1.0f * sh[i];
	}

	/* reset the buffers; */
	reset32f(actBuf, ITERATIONS);
	reset32f(senBuf, ITERATIONS);

	i = fir32fInit(&pFir, phLen, ph);

	/* calculate the primary signal */
	fir32fBlockFilter(&pFir, reference, primary, ITERATIONS);

	/* Create and initialize the ADJLMS */
#ifndef STATIC
		i = fxlms32fInit(&fxlms,len,mu,sh,shLen,se,sheLen);
#else
		i = fxlms32fInitStatic(&fxlms,len,mu,sh,shLen,se,sheLen,pCof,pDat);
#endif
		
	if (fxlms32fGetFilterLength(&fxlms) == 0){
		printf("Error creating ADJLMS filter\n");
		exit(-1);
	}

#ifdef FILTERONLY
	i = fxlms32fSetCoef(&fxlms, ph, 0, len, 0);
	printf("coef copied = %d\n",i);
	dispR32f(fxlms.coef, i);
#endif

	start = clock();
	duration = (double) 0.0;

	for (i=0; i<ITERATIONS;i++)	{
		
#ifdef FILTERONLY
		if (i < 1500){
			/* filter only */
			actBuf[i] = fxlms32fFilterOnly(&fxlms, reference[i]);
			senBuf[i] = primary[i] + actBuf[i];
		}
		else
#endif
		{
			/* filter and update the filter coefficients */
			actBuf[i] = fxlms32fFilterUpdate(&fxlms, reference[i], primary[i]);
			senBuf[i] = fxlms32fGetLastSensorSample(&fxlms);
		}

#ifdef RESET
		if (i == 2000){
			/* reset */
			fxlms32fReset(&fxlms);
		}
#endif

#ifdef RESIZE
#ifndef STATIC
		if (i == 3000){
			/* resize */
			j = fxlms32fResize(&fxlms, 8);
			if (j != 0) {
				printf("Couldn't resize\n");
				exit(-1);
			}
			dispR32f(fxlms.coef, fxlms32fGetFilterLength(&fxlms));
		}
#endif
#endif
	} 

	finish = clock();
	duration = (double)(finish - start) ;

	/* Open files */
	actFile	= fopen (actFileName, "wt") ;
	if (actFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing",actFileName); 
		return(-2);
	}

	senFile	= fopen (senFileName, "wt") ;
	if (senFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", senFileName); 
		return(-3);
	}
	
	primFile	= fopen (primFileName, "wt") ;
	if (senFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", primFileName); 
		return(-3);
	}

	/* write output and error samples to streams */
	for (i=0; i<ITERATIONS;i++)	{
		fprintf(senFile, "%f\n", senBuf[i]);
		fprintf(actFile, "%f\n", actBuf[i]);
		fprintf(primFile, "%f\n", primary[i]);
	}
	fclose(senFile);
	fclose(actFile);

#ifdef PRINTCOEF
	/* save and print the filter coefficients */
	
	firFile	= fopen (firFileName, "wt") ;
	if (firFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", firFileName); 
		return(-4);
	}

	coef = fxlms32fGetCoefPtr(&fxlms);
	i = fxlms32fGetCoef(&fxlms, fir, 0, len, 0);

	dispR32f(coef, fxlms32fGetFilterLength(&fxlms));
	fdispR32f(firFile, fir, fxlms32fGetFilterLength(&fxlms));

	fclose(firFile);
#endif


#ifdef DELAYLINE
	/* test delay line Set and Get */
	fxlms32fGetDelayLine(&fxlms, fir, 0, 10, 0);
	dispR32f(fir, fxlms32fGetFilterLength(&fxlms));
//	fxlms32fResetDelayLine(&fxlms);
	fxlms32fGetDelayLine(&fxlms, fir, 0, 10, 0);
	dispR32f(fir, fxlms32fGetFilterLength(&fxlms));

	i = fxlms32fSetDelayLine(&fxlms, ph, 0, 5, 0);

	i = fxlms32fGetDelayLine(&fxlms, fir, 0, 10, 0);
	dispR32f(fir, fxlms32fGetFilterLength(&fxlms));

#endif

#ifdef MACROS
	printf("FilterLength= %d, stepSize= = %f, coefPtr=%d, secPtr=%d, estSecPtr=%d, secLength=%d, estSecLength=%d\n", 
		fxlms32fGetFilterLength(&fxlms),  
		fxlms32fGetStepSize(&fxlms),
		fxlms32fGetCoefPtr(&fxlms),
		fxlms32fGetSecPathPtr(&fxlms),
		fxlms32fGetEstSecPathPtr(&fxlms),
		fxlms32fGetSecPathLength(&fxlms),
		fxlms32fGetEstSecPathLength(&fxlms) );

	fxlms32fSetStepSize(&fxlms, 0.111f);
	fxlms32fSetSecPathPtr(&fxlms, NULL);
	fxlms32fSetEstSecPathPtr(&fxlms, NULL);
	fxlms32fSetSecPathLength(&fxlms, 0);
	fxlms32fSetEstSecPathLength(&fxlms, 0);

	printf("FilterLength= %d, stepSize= = %f, coefPtr=%d, secPtr=%d, estSecPtr=%d, secLength=%d, estSecLength=%d\n", 
		fxlms32fGetFilterLength(&fxlms),  
		fxlms32fGetStepSize(&fxlms),
		fxlms32fGetCoefPtr(&fxlms),
		fxlms32fGetSecPathPtr(&fxlms),
		fxlms32fGetEstSecPathPtr(&fxlms),
		fxlms32fGetSecPathLength(&fxlms),
		fxlms32fGetEstSecPathLength(&fxlms) );
#endif


#ifndef STATIC
	fxlms32fFree(&fxlms);
#endif

  printf("\nElapsed Time    in Seconds\n");
  printf(  "FXLMS          %10.10f\n",  (float)duration/ CLOCKS_PER_SEC );

	return 0;
}


