/*  ********************************************************************
	TESTLMS.C
	Test application for the LMS filter
    ******************************************************************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#include <time.h>
#include "asptlms32f.h"
#include "asptfir32f.h"
#include "asptdisplay32f.h"

/****************/
/* Test Options */
/****************/

//#define STATIC
//#define RESIZE
//#define RESET
//#define FILTERONLY
#define PRINTCOEF
//#define MACROS
//#define LC
#define LMS
//#define SGNERR
//#define SGNREG
//#define SGNSGN

#define ITERATIONS 5000
#define NF 10		/* adaptive filter length */
#define HL 8        /* unknown filter length  */

int main(int argc, char* argv[])
{

	int		i,j,k;
	asptLms32f  lms ;			/* adaptive filter          */
	asptFir32f hFir;			/* unknown filter           */
	int		Nf = NF;			/* filter length		    */
	float	mu = 0.1f;			/* step size                */

	FILE	*outFile;			/* output signal log file	*/
	FILE	*errFile;			/* error signal log file	*/
	FILE	*inpFile;			/* input signal log file	*/
	FILE	*desFile;			/* desired signal log file	*/

	float	inpBuf[ITERATIONS];
	float	desBuf[ITERATIONS];
	float	errBuf[ITERATIONS];
	float	outBuf[ITERATIONS];
	float	unknown[]={.1f, .3f, -.4f, .7f, -.3f, .9f, .5f, .2f};

	float	fir[4*NF];			/* buffer for printing purposes */

	char	*outFileName="out.txt";
	char	*errFileName="err.txt";
	char	*inpFileName="inp.txt";
	char	*desFileName="des.txt";

	clock_t start, finish;
	double  duration;

#ifdef PRINTCOEF
	FILE	*firFile;			/* FIR coef. file           */	
	char	*firFileName="cof.txt";
#endif
	
#ifdef STATIC
	float	pCof[NF];		/* filterLength */
	float	pDat[NF];		/* filterLength */	
#endif


	/* create the unknown filter */
	i = fir32fInit(&hFir, HL, unknown);
	if (fir32fGetFilterLength(&hFir)==0){
		printf("Couldn't create unknown filter\n");
		exit(-1);
	}

	/* reset the buffers; */
	reset32f(inpBuf, ITERATIONS);
	reset32f(desBuf, ITERATIONS);
	reset32f(outBuf, ITERATIONS);
	reset32f(errBuf, ITERATIONS);	

	/* Create and initialize the asptBfdaf filter */
#ifndef STATIC
		i = lms32fInit(&lms,Nf,mu);
#else
		i = lms32fInitStatic(&lms,Nf,mu,pCof,pDat);
#endif	
	
	if (i < 0){
		printf("Error creating asptNlms filter\n");
		exit(-1);
	}


#ifdef FILTERONLY
	j= lms32fSetCoef(&lms, unknown, 0, Nf,0);
	printf("%d coef have been set.\n",j);
	j = lms32fGetCoef(&lms, fir, 0, Nf, 0);
	dispR32f(fir,j);	
#endif

	/* fill in the input buffer */
	for (k=0; k<ITERATIONS; k++){		
		inpBuf[k] = (float) rand()/(float)RAND_MAX - 0.5f;;	
	}

	/* calculate the desired signal */
	fir32fBlockFilter(&hFir, inpBuf, desBuf, ITERATIONS);

	start = clock();
	duration = (double) 0.0;

	for (i=0; i<ITERATIONS;i++)	{
#ifdef LC
		insertSampleLinBuf32f(inpBuf[i],lms.delayLine,lms32fGetDelayLineLength(&lms));
#endif
#ifdef FILTERONLY
		if (i < 2000)	{	
			/* filter but no update */
#ifdef LC

#else
			outBuf[i] = lms32fFilterOnly(&lms, inpBuf[i]);
#endif
			errBuf[i] = desBuf[i] - outBuf[i];
		}
		else
#endif
		{
			/* filter and update the filter coefficients */
#ifdef LMS
#ifdef LC
			outBuf[i] = lms32fLCFilterUpdate(&lms,  desBuf[i]);
#else
			outBuf[i] = lms32fFilterUpdate(&lms, inpBuf[i], desBuf[i]);
#endif
#endif
#ifdef SGNERR
			lms32fSetStepSize(&lms,0.01f);
#ifdef LC
			outBuf[i] = lms32fLCFilterUpdateSignErr(&lms,  desBuf[i]);
#else
			outBuf[i] = lms32fFilterUpdateSignErr(&lms, inpBuf[i], desBuf[i]);
#endif

#endif
#ifdef SGNREG
#ifdef LC
			outBuf[i] = lms32fLCFilterUpdateSignReg(&lms,  desBuf[i]);
#else
			outBuf[i] = lms32fFilterUpdateSignReg(&lms, inpBuf[i], desBuf[i]);
#endif
#endif
#ifdef SGNSGN
			lms32fSetStepSize(&lms,0.01f);
#ifdef LC
			outBuf[i] = lms32fLCFilterUpdateSignSign(&lms,  desBuf[i]);
#else
			outBuf[i] = lms32fFilterUpdateSignSign(&lms, inpBuf[i], desBuf[i]);
#endif
#endif
			errBuf[i] = lms32fGetLastErrorSample(&lms);
		}



#ifdef RESET
		if (i == 3000)	{
			lms32fReset(&lms);
		}
#endif

#ifdef RESIZE
#ifndef STATIC
		if (i == 3000){
			Nf = 8;
			j = lms32fResize(&lms, Nf);
			printf("RESIZE returned %d\n RESIZE::",j);
			j = lms32fGetFilterLength(&lms);
			k = lms32fGetCoef(&lms, fir, 0, j, 0);
			dispR32f(fir, k);
		}

#endif
#endif
	} 

		
	finish = clock();
	duration = (double)(finish - start) ;

	/* Open files */
	inpFile	= fopen (inpFileName, "wt") ;
	if (inpFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing",inpFileName); 
		return(-2);
	}

	desFile	= fopen (desFileName, "wt") ;
	if (desFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", desFileName); 
		return(-3);
	}

	outFile	= fopen (outFileName, "wt") ;
	if (outFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", outFileName); 
		return(-3);
	}
	
	errFile	= fopen (errFileName, "wt") ;
	if (errFile==NULL)	{
		fprintf(stdout,"Can't open file %s for reading", errFileName); 
		return(-3);
	}

		/* write samples to streams */
		for (j=0; j<ITERATIONS;j++)	{
			fprintf(inpFile, "%+6.5f\n", inpBuf[j]);
			fprintf(desFile, "%+6.5f\n", desBuf[j]);
			fprintf(outFile, "%+6.5f\n", outBuf[j]);		
			fprintf(errFile, "%+6.5f\n", errBuf[j]);
		}
	fclose(inpFile);
	fclose(desFile);
	fclose(errFile);
	fclose(outFile);

#ifdef PRINTCOEF
	/* save and print the filter coefficients */
	firFile	= fopen (firFileName, "wt") ;
	if (firFile==NULL){
		fprintf(stdout,"Can't open file %s for writing", firFileName); 
		return(-4);
	}

	dispR32f(lms.coef, Nf);
	lms32fGetCoef(&lms, fir,0, Nf,0);
	dispR32f(fir, lms32fGetFilterLength(&lms));
	fdispC32f(firFile, fir, lms32fGetFilterLength(&lms));

	fclose(firFile);
#endif

#ifdef MACROS
	printf("FL=%d, SS=%f, coefPtr=%d,\n", 
	lms32fGetFilterLength(&lms),  
	lms32fGetStepSize(&lms),
	lms32fGetCoefPtr(&lms));

#ifndef STATIC
	lms32fResize(&lms,12);
#endif

	lms32fSetStepSize(&lms, 0.111f);

	printf("FL=%d, SS=%f, coefPtr=%d,\n", 
	lms32fGetFilterLength(&lms),  
	lms32fGetStepSize(&lms),
	lms32fGetCoefPtr(&lms));
#endif

	printf("Filter Length = %d\n",lms32fGetFilterLength(&lms));
#ifndef STATIC
	lms32fFree(&lms);
#endif
	fir32fFree(&hFir);

   printf("\nElapsed Time      in Seconds\n");
   printf(  "LMS               %10.10f\n", (float)duration/ CLOCKS_PER_SEC );


	return 0;
}



