/*  ********************************************************************
	TESTRLSS.C
	Test application for the RLS filter
    ******************************************************************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */


#include <time.h>
#include "asptrls64f.h"
#include "asptfir64f.h"
#include "asptdisplay64f.h"

/****************/
/* Test Options */
/****************/

//#define STATIC
//#define RESIZE
//#define RESET
//#define FILTERONLY
#define PRINTCOEF
//#define MACROS
#define LC

#define ITERATIONS 5000
#define NF 10		/* adaptive filter length */
#define HL 8        /* unknown filter length  */

int main(int argc, char* argv[])
{

	int		i,j,k;
	asptRls64f  rls ;			/* adaptive filter          */
	asptFir64f hFir;			/* unknown filter           */
	int		Nf = NF;			/* filter length		    */
	double	mu = 0.98f;			/* step size                */

	FILE	*outFile;			/* output signal log file	*/
	FILE	*errFile;			/* error signal log file	*/
	FILE	*inpFile;			/* input signal log file	*/
	FILE	*desFile;			/* desired signal log file	*/

	double	inpBuf[ITERATIONS];
	double	desBuf[ITERATIONS];
	double	errBuf[ITERATIONS];
	double	outBuf[ITERATIONS];
	double	unknown[]={.1f, .3f, -.4f, .7f, -.3f, .9f, .5f, .2f};

	double	fir[4*NF];			/* buffer for printing purposes */

	char	*outFileName="out.txt";
	char	*errFileName="err.txt";
	char	*inpFileName="inp.txt";
	char	*desFileName="des.txt";

	clock_t start, finish;
	double  duration;

#ifdef PRINTCOEF
	FILE	*firFile;			/* FIR coef. file           */	
	char	*firFileName="cof.txt";
#endif
	
#ifdef STATIC
	double	pCof[NF];			/* filterLength */
	double	pDat[NF*(NF+2)];	/* NF*(NF+2) or rsum(1,NF)+2*NF */	
#endif


	/* create the unknown filter */
	i = fir64fInit(&hFir, HL, unknown);
	if (fir64fGetFilterLength(&hFir)==0){
		printf("Couldn't create unknown filter\n");
		exit(-1);
	}

	/* reset the buffers; */
	reset64f(inpBuf, ITERATIONS);
	reset64f(desBuf, ITERATIONS);
	reset64f(outBuf, ITERATIONS);
	reset64f(errBuf, ITERATIONS);	

	/* Create and initialize the asptRls filter */
#ifndef STATIC
		i = rls64fInit(&rls,Nf,mu, 0.1f);
#else
		i = rls64fInitStatic(&rls,Nf,mu,0.1f,pCof,pDat);
#endif	
	
	if (i < 0){
		printf("Error creating asptRls filter\n");
		exit(-1);
	}


#ifdef FILTERONLY
	j= rls64fSetCoef(&rls, unknown, 0, Nf,0);
	printf("%d coef have been set.\n",j);
	j = rls64fGetCoef(&rls, fir, 0, Nf, 0);
	dispR64f(fir,j);	
#endif

	/* fill in the input buffer */
	for (k=0; k<ITERATIONS; k++){		
		inpBuf[k] = (double) rand()/(double)RAND_MAX - 0.5f;;	
	}

	/* calculate the desired signal */
	fir64fBlockFilter(&hFir, inpBuf, desBuf, ITERATIONS);

	start = clock();
	duration = (double) 0.0;

	for (i=0; i<ITERATIONS;i++)	{

#ifdef LC
			insertSampleLinBuf64f(inpBuf[i],rls.delayLine,rls64fGetDelayLineLength(&rls));
#endif

#ifdef FILTERONLY
		if (i < 2000)	{	
			/* filter but no update */
#ifdef LC
			outBuf[i] = rls64fLCFilterOnly(&rls);
#else
			outBuf[i] = rls64fFilterOnly(&rls, inpBuf[i]);
#endif

			errBuf[i] = desBuf[i] - outBuf[i];
		}
		else
#endif
		{
			/* filter and update the filter coefficients */
#ifndef LC
			outBuf[i] = rls64fFilterUpdate(&rls, inpBuf[i], desBuf[i]);
#else
			outBuf[i] = rls64fLCFilterUpdate(&rls,  desBuf[i]);
#endif
			errBuf[i] = rls64fGetLastErrorSample(&rls);
		}



#ifdef RESET
		if (i == 3000)	{
			rls64fReset(&rls);
		}
#endif

#ifdef RESIZE
#ifndef STATIC
		if (i == 3000){
			Nf = 8;
			j = rls64fResize(&rls, Nf);
			printf("RESIZE returned %d\n RESIZE::",j);
			j = rls64fGetFilterLength(&rls);
			k = rls64fGetCoef(&rls, fir, 0, j, 0);
			dispR64f(fir, k);
		}

#endif
#endif
	} 

		
	finish = clock();
	duration = (double)(finish - start) ;

	/* Open files */
	inpFile	= fopen (inpFileName, "wt") ;
	if (inpFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing",inpFileName); 
		return(-2);
	}

	desFile	= fopen (desFileName, "wt") ;
	if (desFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", desFileName); 
		return(-3);
	}

	outFile	= fopen (outFileName, "wt") ;
	if (outFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", outFileName); 
		return(-3);
	}
	
	errFile	= fopen (errFileName, "wt") ;
	if (errFile==NULL)	{
		fprintf(stdout,"Can't open file %s for writing", errFileName); 
		return(-3);
	}

		/* write samples to streams */
		for (j=0; j<ITERATIONS;j++)	{
			fprintf(inpFile, "%+6.5f\n", inpBuf[j]);
			fprintf(desFile, "%+6.5f\n", desBuf[j]);
			fprintf(outFile, "%+6.5f\n", outBuf[j]);		
			fprintf(errFile, "%+6.5f\n", errBuf[j]);
		}
	fclose(inpFile);
	fclose(desFile);
	fclose(errFile);
	fclose(outFile);

#ifdef PRINTCOEF
	/* save and print the filter coefficients */
	firFile	= fopen (firFileName, "wt") ;
	if (firFile==NULL){
		fprintf(stdout,"Can't open file %s for writing", firFileName); 
		return(-4);
	}

	dispR64f(rls.coef, Nf);
	rls64fGetCoef(&rls, fir,0, Nf,0);
	dispR64f(fir, rls64fGetFilterLength(&rls));
	fdispC64f(firFile, fir, rls64fGetFilterLength(&rls));

	fclose(firFile);
#endif

#ifdef MACROS
	printf("FL=%d, SS=%f, coefPtr=%d,\n", 
	rls64fGetFilterLength(&rls),  
	rls64fGetForgetFactor(&rls),
	rls64fGetCoefPtr(&rls));

#ifndef STATIC
	rls64fResize(&rls,12);
#endif

	rls64fSetForgetFactor(&rls, 0.991f);

	printf("FL=%d, SS=%f, coefPtr=%d,\n", 
	rls64fGetFilterLength(&rls),  
	rls64fGetForgetFactor(&rls),
	rls64fGetCoefPtr(&rls));
#endif

#ifndef STATIC
	rls64fFree(&rls);
#endif
	fir64fFree(&hFir);

   printf("\nElapsed Time      in Seconds\n");
   printf(  "RLS               %10.10f\n",  (double)duration/ CLOCKS_PER_SEC );


	return 0;
}



