/*  ********************************************************************
	TESTRLSLBPEF.C 
	simple system identification test application for the RLSLBPEF 
	adaptive algorithm.
    ******************************************************************** */

/*  ********************************************************************
    Copyright (c) DSP ALGORITHMS 2000-2002.
    All rights reserved. The code herein is and remains the Property
    of DSP ALGORITHMS. Restricted rights to use, modify, or disclose
    are granted through a license agreement. Please read and understand
    the license terms and conditions carefully before using this code.
    ******************************************************************** */

#include <time.h>
#include "asptdisplay64f.h"
#include "asptrlslbpef64f.h"



/****************/
/* Test Options */
/****************/

//#define STATIC
//#define RESIZE
//#define RESET
//#define FILTERONLY
#define PRINTCOEF
//#define MACROS
//#define SPEED

#define ITERATIONS 5000
#define NF 8

int main(int argc, char* argv[])
{
	asptRlslbpef64f	rlsl ;		// adaptive iir
	int		len = NF;			// filter length
	double	mu = 0.998f;
	int		i, j=0, k=0;
	double	fir[10];
	double	time ;
	double	fs	 = 1000.0f;		// sampling frequency
	double	ts   = 1/fs;		// sampling period

	FILE	*outFile;	/* output signal log file */
	FILE	*errFile;	/* error signal log file */

	double	inputBuf[ITERATIONS];
	double	errorBuf[ITERATIONS];
	double	outputBuf[ITERATIONS];

	char	*outFileName="out.txt";
	char	*errFileName="err.txt";

	clock_t start, finish;
	double  duration;

#ifdef PRINTCOEF
	FILE	*firFile;			/* FIR coef. file   */	
	char	*firFileName="cof.txt";
#endif
	
#ifdef STATIC
	double	pCof[NF*3];			
	double	pDat[(NF)*6+1];		
#endif
	

	// make input signal = a random input signal + sinusoidal of 50 Hz sampled at 1 kHz
	for (i=0, time = 0; i<ITERATIONS; i++)
	{ 
		time += ts;
		inputBuf[i] = (double) 2.0f * (rand()/(double)RAND_MAX - 0.5f);
		inputBuf[i] += (double) (1.0 * cos(PIX2 * 50.0 *time));
		inputBuf[i] += (double) (1.0 * cos(PIX2 * 200.0 *time));
		outputBuf[i] = 0.0f;
		errorBuf[i]  = 0.0f;
	}


	/* create rlsl filter */
#ifndef STATIC
		i = rlslbpef64fInit(&rlsl, len, mu);	
#else
		i = rlslbpef64fInitStatic(&rlsl, len, mu, pCof, pDat);
#endif	
	if (rlslbpef64fGetFilterLength(&rlsl) == 0)
	{
		printf("Error creating asptRlslbpef filter\n");
		exit(-1);
	}

	start = clock();
	duration = (double) 0.0;

	for (i=0; i<ITERATIONS;i++)	{

		
#ifdef FILTERONLY
		if (i > 1000)	// one second
		{
			outputBuf[i] = rlslbpef64fFilterOnly(&rlsl, inputBuf[i]);
			errorBuf[i]  = inputBuf[i] - outputBuf[i];
		}
		else
#endif
		{// filter and update the filter coefficients
			outputBuf[i] = rlslbpef64fFilterUpdate(&rlsl, inputBuf[i]);
			errorBuf[i]  = rlslbpef64fGetLastErrorSample(&rlsl);

		}

#ifdef RESET
		if (i == 4900){
			rlslbpef64fReset(&rlsl);
		}
#endif
#ifdef RESIZE
#ifndef STATIC
		if (i == 3000)	{// resize
			len = 10;
			j = rlslbpef64fResize(&rlsl, 10);
			j = rlslbpef64fGetCoef(&rlsl, fir, 0, len, 0);
			dispR64f(fir, j);
		}
#endif
#endif
	} 

	finish = clock();
	duration = (double)(finish - start) ;

	// Open files
	outFile	= fopen (outFileName, "wt") ;
	if (outFile==NULL)
	{
		fprintf(stdout,"Can't open file %s for writing",outFileName); 
		return(-2);
	}

	errFile	= fopen (errFileName, "wt") ;
	if (errFile==NULL)
	{
		fprintf(stdout,"Can't open file %s for writing", errFileName); 
		return(-3);
	}
#ifndef SPEED	
	// write output and error samples to streams
	for (i=0; i<ITERATIONS;i++)
	{
		fprintf(errFile, "%f\n", errorBuf[i]);
		fprintf(outFile, "%f\n", outputBuf[i]);
	}
#endif

	fclose(errFile);
	fclose(outFile);

#ifdef PRINTCOEF
	firFile	= fopen (firFileName, "wt") ;
	if (firFile==NULL)
	{
		fprintf(stdout,"Can't open file %s for writing", firFileName); 
		return(-4);
	}

	i = rlslbpef64fGetCoef(&rlsl, fir, 0, len, 0);
	printf("LINEAR COMBINER::\n");
	fdispR64f(firFile,fir,len);
	dispR64f(fir,len);

	i = rlslbpef64fGetForwardParcors(&rlsl, fir, 0, len, 0);
	printf("FORWARD PARKOR::\n");
	fdispR64f(firFile,fir,len);
	dispR64f(fir,len);

	i = rlslbpef64fGetBackwardParcors(&rlsl, fir, 0, len, 0);
	printf("BACKWARD PARKOR::\n");
	fdispR64f(firFile,fir,len);
	dispR64f(fir,len);

	fclose(firFile);
#endif

#ifdef MACROS

	printf("L=%d, FF=%f,  F=%d, DLPtr=%d, PFPtr=%d, PBPtr=%d, CofPtr=%d\n", 
		rlslbpef64fGetFilterLength(&rlsl),  
		rlslbpef64fGetForgetFactor(&rlsl), 
		rlslbpef64fGetFirFlag(&rlsl),
		rlslbpef64fGetDelayLinePtr(&rlsl),
		rlslbpef64fGetForwardParcorPtr(&rlsl),
		rlslbpef64fGetBackwardParcorPtr(&rlsl),
		rlslbpef64fGetCoefPtr(&rlsl));

#ifndef STATIC
	rlslbpef64fResize(&rlsl,12);
#endif
	rlslbpef64fSetForgetFactor(&rlsl, 0.111f);
	rlslbpef64fSetFirFlag(&rlsl, 2);

	printf("L=%d, FF=%f,  F=%d, DLPtr=%d, PFPtr=%d, PBPtr=%d, CofPtr=%d\n", 
		rlslbpef64fGetFilterLength(&rlsl),  
		rlslbpef64fGetForgetFactor(&rlsl), 
		rlslbpef64fGetFirFlag(&rlsl),
		rlslbpef64fGetDelayLinePtr(&rlsl),
		rlslbpef64fGetForwardParcorPtr(&rlsl),
		rlslbpef64fGetBackwardParcorPtr(&rlsl),
		rlslbpef64fGetCoefPtr(&rlsl));
#endif

#ifndef STATIC
	rlslbpef64fFree(&rlsl);
#endif

   printf("\nElapsed Time      in Seconds\n");
   printf(  "RLSLBPEF          %10.10f\n",  (double)duration/ CLOCKS_PER_SEC );

	return 0;
}


